﻿#ifndef __UDBAUTH_AUTHEVENT___H__
#define __UDBAUTH_AUTHEVENT___H__

#pragma once 

#include <string>
#include <sstream>

#include "json_lib.h"
#include "int_types.h"

/**
 * 概述：Auth库回调事件
 * 
 * @version 1.0
 * @author weitianpeng@yy.com
 * @time 2015年9月9日 下午6:06:21
 */

using namespace std;

namespace udbauth {


	class AuthError {
	
	public:
	
		AuthError(int code, std::string description) {
			this->_code = code;
			this->_description = description;
		}

		/** 获取错误码*/
		int code() {
			return _code;
		}

		/** 获取错误码描述，也可以用{@link #toString()}代替*/
		std::string description() {
			return _description;
		}

		std::string toString() {
			return _description;
		}

		static AuthError Json_Error()
		{
			static AuthError json_error(12,"JSON格式错误");
			return json_error;
		}

	private:
		/*错误码*/
		int _code;
		/*错误码描述*/
		std::string _description;

	};

    

	/** 回调事件类型*/
	class EventOpCmd {
	public:
		/** 回调事件类型：非法事件*/
		const static int OP_INVALID_EVENT = -1;
		/** 回调事件类型：登录事件*/
		const static int OP_LOGIN_EVENT = 1;
		/** 回调事件类型：下发短信*/
		const static int OP_SEND_SMS_EVENT = 2;
		/** 回调事件类型：刷新图片验证码*/
		const static int OP_REFRESH_PIC_EVENT = 3;
		/** 回调事件类型：查询请求事件*/
		const static int OP_QUERY_EVENT = 4;
		/** 回调事件类型：匿名登录*/
		const static int OP_ANONYMOUS_EVENT = 5;
		/** 回调事件类型：操作超时*/
		const static int OP_TIME_OUT = 6;
		/** 回调事件类型：自动刷新凭证*/
		const static int OP_CREDIT_RENEW = 7;
		/** 回调事件类型：验证手机（或账号）和短信验证码是否正确的响应*/
		const static int OP_VERIFY_SMSCODE = 8;
		/** 回调事件类型：注册新用户的响应*/
		const static int OP_REGISTER = 9;
		/** 回调事件类型：改密检测的响应*/
		const static int OP_CHECK_MODPWD = 10;
		/** 回调事件类型：改密的响应*/
		const static int OP_MODPWD = 11;
		/** 回调事件类型：注册检测的响应*/
		const static int OP_CHECK_REG = 12;
		/** 回调事件类型：用户检测的响应*/
		const static int OP_CHECK_USER = 13;
		/** 回调事件类型：交换第三方票据响应*/
		const static int OP_CHECK_THIRDTOKEN = 17;
		/** 回调事件类型：获取YY客户端用户信息*/
		const static int OP_GETQUICK_YYLIST = 100;
		/** 回调事件类型：获取YY客户端用户票据*/
		const static int OP_GETQUICK_YYTICKET = 101;
	};

	/** 业务收到响应包后的处理方式操作类型*/
	class UIAction {
	public:
		/** 业务处理类型：成功*/
		const static int SUCCESS = 0;
		/** 业务处理类型：表示操作失败，业务需要显示后端返回的描述*/
		const static int FAILED = 1;
		/** 业务处理类型：表示需要下一步验证，业务应该跳转到下一步验证*/
		const static int NEXT_VERIFY = 2;
		/** 业务处理类型：表示凭证已无效，业务需要删除凭证，并让用户重新登录*/
		const static int CREDIT_INVALID = 3;
		/** 业务处理类型：表示动态校验错误（图片、滑动、短信、手机令牌、硬件令牌）*/
		const static int VERIFY_FAILED = 4;
		/** 业务处理类型：表示服务端还没收到上行短信，需重试*/
		const static int SERVER_HAS_NOT_RECEIVED_SMS = 5;
		/** 业务处理类型：跳转到url*/
		const static int JUMP_TO_URL = 6;
	};

	/** 回调事件类型*/
	class NextVerifyDataType
	{
	public:
		/** 0 data没有内容
		 * 1 二进制图片
		 * 2 文本
		 * 3 图片url
		 * 4 网页url
		 * 5 嵌入到浏览器的js */
		const static int DATA_TYPE_NO_DATA = 0;
		const static int DATA_TYPE_PIC_BIN = 1;
		const static int DATA_TYPE_TEXT = 2;
		const static int DATA_TYPE_PIC_URL = 3;
		const static int DATA_TYPE_WEB_URL = 4;
		const static int DATA_TYPE_WEB_JS = 5;
	};

	/**
	 * 概述：二次验证策略的内容
	 * 
	 * @version 1.0
	 * @author weitianpeng@yy.com
	 * @time 2015年11月20日 下午2:45:01
	 */

	class NextVerify  {
	public:
		
		/** 二次验证策略类型，见{@link STRATEGY}类声明常量 */
		int strategy;// STRATEGY::NONE  // TODO
		/** 选择界面显示的标题，如：短信验证码、图片验证码……*/
		std::string selectTitle ;
		/** 界面显示的标题，如：你绑定的手机号是*/
		std::string promptTitle ;
		/** 
		 * 界面显示提示内容，如：186******50<br/>
		 * <p>当策略类型为上行短信时，该值为json字符串<br/>
		 * {<br/>
		 *   "gateway":"上行短信目标号码",<br/>
		 *   "code":"上行短信内容"<br/>
		 * }<br/>
		 * </p>
		 */
		std::string promptContent ;
		/*
		 * 见{@link NextVerifyDataType}类声明常量
		 */
		int dataType;

		/** 
		 * 如果是图片验证，这里是经过BASE64编码后的验证码图片<br/>
		 * 如果是Web交互验证，这里的经过BASE64编码后的js代码
		 */
		std::string data;
		/*
		*如果是HTML5验证，这里输出HTML5页面的宽和高
		*/
		int box_high;
		int box_length;

		NextVerify(): strategy(0),selectTitle(""),promptTitle(""),promptContent(""),data(""),box_high(0),box_length(0){
		}
	};

	/**
	 * 概述：第三方登录返回的信息
	 */
	class ThirdPartyInfo {
	public:
		/** 用户唯一标识*/
		std::string uid;
		/** 用户昵称*/
		std::string nickname;
		/** 用户头像地址*/
		std::string imageUrl;
		/** 用户性别*/
		std::string gender;
		/*扩展*/
		std::string unionid;
		/** openid对应的yyuid，账号合并需求*/
		std::string openid_yyuid;
		/** 预留字段暂时没用*/
		std::string reserve1;
		std::string reserve2;
		std::string reserve3;
	};

	/**
	 * 概述：回调事件的基类
	 */
	class AuthBaseEvent {

	public:

		/** 回调事件的类型, 参考 {@link EventOpCmd}*/
		int OP_CMD ;

		/** JSON版本号*/
		int json_ver;

		/** 业务JSON内容*/
		std::string json ;

		/** 最外层JSON*/
		CSJson::Value mRootJsonObject;

		AuthBaseEvent():OP_CMD(EventOpCmd::OP_INVALID_EVENT),json_ver(1),json("")
		{
		}

		AuthBaseEvent(const AuthBaseEvent* base){
			this->json = base->json;
			this->OP_CMD = base->OP_CMD;
			this->json_ver = base->json_ver;
			this->mRootJsonObject = base->mRootJsonObject;
		}


		/** 给子类用的，获取上层JSON对象*/
		CSJson::Value getParentJsonObject() {
			return mRootJsonObject;
		}

		/** Json返回的回调事件的类型*/
		int opCmd() {
			return OP_CMD;
		}

		/** 返回JSON协议版本号*/
		int getJsonVer() {
			return json_ver;
		}

		std::string marshall()
		{
			char strcmd[10] = {0};
			sprintf(strcmd, "%d", OP_CMD);
			char strjver[10] = {0};
			sprintf(strjver, "%d", json_ver);

            mRootJsonObject["op_cmd"] = CSJson::Value(strcmd);
            mRootJsonObject["json_ver"] = CSJson::Value(strjver);

			CSJson::FastWriter fast_writer;
			json=fast_writer.write(mRootJsonObject);
            return json;
		}

		
		/** 传递JSON字节数组*/
		void unmarshall(string jsonstr) {
			try {
				//JSON反序列化 
				json = jsonstr;

				CSJson::Reader jreader;
				// 只能做一层的对象解释；
				if (jreader.parse(json.c_str(), mRootJsonObject)) 
				{
					if(mRootJsonObject.isMember("op_cmd") ){
						std::string str_op_cmd = mRootJsonObject["op_cmd"].asString();
						OP_CMD = atoi(str_op_cmd.c_str());
					}

					if(mRootJsonObject.isMember("json_ver") ){
						std::string str_json_ver = mRootJsonObject["json_ver"].asString();
						json_ver = atoi(str_json_ver.c_str());
					}
				}
			} catch (exception e) {
				
				OP_CMD = EventOpCmd::OP_INVALID_EVENT;
				/*
				mRootJsonObject = null;
				Log.d("AuthEvent unmarshall", "JSON数据格式错误！");
				AuthSDK.sendClientLogRequest("", 1, "AuthBaseEvent.unmarshall() JSONException std::string size is "
						+ (buf == null ? 0 : buf.length) + "content is " + new std::string(buf));*/
			}
		}
	};



	/**
	 * 概述：登录回调事件
	 */
	class LoginEvent : public AuthBaseEvent {

	public:

		/** 返回码*/
		int errCode;
		/** 返回码描述*/
		std::string description;
		/** UID*/
		std::string uid;
		/** YY号*/
		std::string yyid;
		/** 通行证*/
		std::string passport;
		/** 手机号掩码*/
		std::string mobile_mask;
		/** 邮箱掩码*/
		std::string email_mask;
		/** 凭证*/
		std::string credit;
		/** 是否是新注册用户，针对手机短信登录的情况，其它登录无效。*/
		int newUser ;
		/** 当需要二次验证时，这里是后端返回的一个或多个二次验证类型，用户可以选择其中一个进行验证*/
		std::vector<NextVerify> nextVerifies ;
		/** 第三方登录返回的信息*/
		ThirdPartyInfo thirdPartyInfo;

		/** 
		 * 包括如下这些：
		 * <li>{@link UIAction#SUCCESS}</li>
		 * <li>{@link UIAction#FAILED}</li>
		 * <li>{@link UIAction#NEXT_VERIFY}</li>
		 * <li>{@link UIAction#CREDIT_INVALID}</li>
		 * <li>{@link UIAction#VERIFY_FAILED}</li>
		 * <li>{@link UIAction#SERVER_HAS_NOT_RECEIVED_SMS}</li>
		 */
		int uiAction;
		/** 上下文。请求和响应的上下文是呼应的。请求的上下文是什么，这里就是什么*/
		std::string context;
		/*url链接*/
		std::string url;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		LoginEvent():errCode(0),description(""),uid(""),yyid(""),passport(""),mobile_mask(""),email_mask(""),credit(""),newUser(0),url("")
		{
			OP_CMD=EventOpCmd::OP_LOGIN_EVENT;
		}

		LoginEvent(const AuthBaseEvent* base):errCode(0),description(""),uid(""),yyid(""),passport(""),mobile_mask(""),email_mask(""),credit(""),newUser(0),url(""),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_LOGIN_EVENT;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errCode);

			char struiAction[16] = {0};
			sprintf(struiAction, "%d", uiAction);

            char strnewUser[16] = {0};
			sprintf(strnewUser, "%d", newUser);

		    mJsonObject["errcode"] = CSJson::Value(strerrcode);
            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["uiaction"] = CSJson::Value(struiAction);
            mJsonObject["uid"] = CSJson::Value(uid);
			mJsonObject["yyid"] = CSJson::Value(yyid);
            mJsonObject["passport"] = CSJson::Value(passport);
            mJsonObject["mobile_mask"] = CSJson::Value(mobile_mask);
            mJsonObject["email_mask"] = CSJson::Value(email_mask);
			mJsonObject["credit"] = CSJson::Value(credit);
            mJsonObject["new_user"] = CSJson::Value(strnewUser);
			mJsonObject["context"] = CSJson::Value(context);
			mJsonObject["url"] = CSJson::Value(url);

            CSJson::Value arr_value(CSJson::arrayValue);
			for(int i = 0; i < nextVerifies.size(); ++i)
			{

                char strstrategy[16] = {0};
    			sprintf(strstrategy, "%d", nextVerifies[i].strategy);

				char strdatatype[16] = {0};
    			sprintf(strdatatype, "%d", nextVerifies[i].dataType);

				CSJson::Value json_nv;
				json_nv["strategy"]         = CSJson::Value(strstrategy);
				json_nv["select_title"]     = CSJson::Value(nextVerifies[i].selectTitle);
				json_nv["prompt_title"]     = CSJson::Value(nextVerifies[i].promptTitle);
				json_nv["prompt_content"]   = CSJson::Value(nextVerifies[i].promptContent);
				json_nv["data_type"]        = CSJson::Value(strdatatype);
				json_nv["data"]             = CSJson::Value(nextVerifies[i].data);
				json_nv["box_high"]			= CSJson::Value(nextVerifies[i].box_high);
				json_nv["box_length"]		= CSJson::Value(nextVerifies[i].box_length);
				arr_value[i] = CSJson::Value(json_nv);
			}

			CSJson::Value json_3rd;
			if(thirdPartyInfo.nickname.size() > 0)
			{
				json_3rd["3rd_uid"]     = CSJson::Value(thirdPartyInfo.uid);
				json_3rd["3rd_nickname"]  = CSJson::Value(thirdPartyInfo.nickname);
				json_3rd["3rd_img"]     = CSJson::Value(thirdPartyInfo.imageUrl);
				json_3rd["3rd_gen"]     = CSJson::Value(thirdPartyInfo.gender);
				json_3rd["3rd_unionid"] = CSJson::Value(thirdPartyInfo.unionid);
				json_3rd["openid_yyuid"] = CSJson::Value(thirdPartyInfo.openid_yyuid);
				json_3rd["reserve1"] = CSJson::Value(thirdPartyInfo.reserve1);
				json_3rd["reserve2"] = CSJson::Value(thirdPartyInfo.reserve2);
				json_3rd["reserve3"] = CSJson::Value(thirdPartyInfo.reserve3);
			}

            mJsonObject["next_verify"]=arr_value;
            mJsonObject["3rd"]=json_3rd;
			mRootJsonObject["login_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(string jsonstr) {
		

			CSJson::Value parent = getParentJsonObject();

			if (!parent.isMember("login_res")) {
				errCode = AuthError::Json_Error().code();
				description =AuthError::Json_Error().description();
				uiAction = UIAction::FAILED;
				uid = "";
				yyid = "";
				passport = "";
				mobile_mask = "";
				email_mask = "";
				credit = "";
				context = "";
				url = "";
				return;
			}
			std::string strData;
			CSJson::Value son = parent["login_res"];

			strData = son["errcode"].asString();
			errCode = atoi(strData.c_str());
			description = son["description"].asString();
			strData = son["uiaction"].asString();
			uiAction = atoi(strData.c_str());
			uid = son["uid"].asString(); 
			yyid = son["yyid"].asString();  
			passport = son["passport"].asString();
			mobile_mask = son["mobile_mask"].asString();
			email_mask = son["email_mask"].asString();
			credit = son["credit"].asString();  
			newUser =  atoi((son["new_user"].asString().c_str())) ;//0表示已注册，1表示未注册
			context = son["context"].asString();  
			url = son["url"].asString();
			nextVerifies.clear();

			if (son.isMember("next_verify") ) {

				CSJson::Value next_verify = son["next_verify"];

				for (CSJson::Value::iterator itr = next_verify.begin(); itr != next_verify.end(); ++itr) {
	                CSJson::Value &nxtVerObj = *itr;

                    NextVerify nv ;
					strData = nxtVerObj["strategy"].asString();
					nv.strategy = atoi(strData.c_str());
					nv.selectTitle = nxtVerObj["select_title"].asString(); 
					nv.promptTitle = nxtVerObj["prompt_title"].asString();  
					nv.promptContent = nxtVerObj["prompt_content"].asString();   
					nv.dataType=  atoi(nxtVerObj["data_type"].asString().c_str());   
					nv.data = nxtVerObj["data"].asString(); 
					strData = nxtVerObj["box_high"].asString();
					nv.box_high = atoi(strData.c_str());
					strData = nxtVerObj["box_length"].asString();
					nv.box_length = atoi(strData.c_str());
					nextVerifies.push_back(nv);
				}
			}

			//解析第三方登录结果 
			if (son.isMember("3rd")) {
				CSJson::Value third = son["3rd"];
				thirdPartyInfo.uid = third["3rd_uid"].asString();
				thirdPartyInfo.nickname = third["3rd_nickname"].asString() ;
				thirdPartyInfo.imageUrl = third["3rd_img"].asString() ;
				thirdPartyInfo.gender = third["3rd_gen"].asString() ;
				if (third.isMember("3rd_unionid") && !third["3rd_unionid"].asString().empty())
				{
					thirdPartyInfo.unionid = third["3rd_unionid"].asString();
				}
				if (third.isMember("openid_yyuid") && !third["openid_yyuid"].asString().empty())
				{
					thirdPartyInfo.openid_yyuid = third["openid_yyuid"].asString();
				}
				if (third.isMember("reserve1") && !third["reserve1"].asString().empty())
				{
					thirdPartyInfo.reserve1 = third["reserve1"].asString();
				}
				if (third.isMember("reserve2") && !third["reserve2"].asString().empty())
				{
					thirdPartyInfo.reserve2 = third["reserve2"].asString();
				}
				if (third.isMember("reserve3") && !third["reserve3"].asString().empty())
				{
					thirdPartyInfo.reserve3 = third["reserve3"].asString();
				}
			}
		}
	};

	/**
	 * 概述：下发短信验证码回调事件
	 */
	class SendSmsEvent : public AuthBaseEvent {
	public:

		/** 返回码*/
		int errCode;
		/** 返回码描述*/
		std::string description;
		/** 当需要二次验证时，这里是后端返回的一个或多个二次验证类型，用户可以选择其中一个进行验证*/
		std::vector<NextVerify> nextVerifies;
		/** 
		 * 包括如下这些：
		 * <li>{@link UIAction#SUCCESS}</li>
		 * <li>{@link UIAction#FAILED}</li>
		 * <li>{@link UIAction#NEXT_VERIFY}</li>
		 */
		int uiAction;
		/** 上下文。请求和响应的上下文是呼应的。请求的上下文是什么，这里就是什么*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		SendSmsEvent():errCode(0),description(""),context(""),uiAction(0)
		{
			OP_CMD=EventOpCmd::OP_SEND_SMS_EVENT;
			nextVerifies.clear();
		}

		SendSmsEvent(const AuthBaseEvent* base):errCode(0),description(""),context(""),uiAction(0),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_SEND_SMS_EVENT;
			nextVerifies.clear();
		}


		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errCode);

			char struiAction[16] = {0};
			sprintf(struiAction, "%d", uiAction);

		    mJsonObject["errcode"] = CSJson::Value(strerrcode);
            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["uiaction"] = CSJson::Value(struiAction);
			mJsonObject["context"] = CSJson::Value(context);

            CSJson::Value arr_value(CSJson::arrayValue);
			for(int i = 0; i < nextVerifies.size(); ++i)
			{

                char strstrategy[16] = {0};
    			sprintf(strstrategy, "%d", nextVerifies[i].strategy);

				char strdatatype[16] = {0};
    			sprintf(strdatatype, "%d", nextVerifies[i].dataType);

				CSJson::Value json_nv;
				json_nv["strategy"]         = CSJson::Value(strstrategy);
				json_nv["select_title"]     = CSJson::Value(nextVerifies[i].selectTitle);
				json_nv["prompt_title"]     = CSJson::Value(nextVerifies[i].promptTitle);
				json_nv["prompt_content"]   = CSJson::Value(nextVerifies[i].promptContent);
				json_nv["data_type"]        = CSJson::Value(strdatatype);
				json_nv["data"]             = CSJson::Value(nextVerifies[i].data);
				json_nv["box_high"]			= CSJson::Value(nextVerifies[i].box_high);
				json_nv["box_length"]		= CSJson::Value(nextVerifies[i].box_length);
				arr_value[i] = CSJson::Value(json_nv);
			}

            mJsonObject["next_verify"]=arr_value;
			mRootJsonObject["sendsms_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(std::string jsonstr) {
		

			CSJson::Value parent = getParentJsonObject();

			if (!parent.isMember("sendsms_res")) {

				errCode = AuthError::Json_Error().code();
				description = AuthError::Json_Error().description();
				uiAction = UIAction::FAILED;
				context = "";
				nextVerifies.clear();
				return;
			}
            CSJson::Value son = parent["sendsms_res"];
			std::string strData;
			strData = son["errcode"].asString();
			errCode = atoi(strData.c_str());
			description = son["description"].asString();
			strData = son["uiaction"].asString();
			uiAction = atoi(strData.c_str());
			context = son["context"].asString();
			nextVerifies.clear();

			if (son.isMember("next_verify") ) {

				CSJson::Value next_verify = son["next_verify"];

				for (CSJson::Value::iterator itr = next_verify.begin(); itr != next_verify.end(); ++itr) {
	                CSJson::Value &nxtVerObj = *itr;

                    NextVerify nv ;
					strData = nxtVerObj["strategy"].asInt();
					nv.strategy = atoi(strData.c_str());
					nv.selectTitle = nxtVerObj["select_title"].asString(); 
					nv.promptTitle = nxtVerObj["prompt_title"].asString();  
					nv.promptContent = nxtVerObj["prompt_content"].asString();    
					nv.data = nxtVerObj["data"].asString();
					strData = nxtVerObj["box_high"].asString();
					nv.box_high = atoi(strData.c_str());
					strData = nxtVerObj["box_length"].asString();
					nv.box_length = atoi(strData.c_str());
					nextVerifies.push_back(nv);
				}
			}

		}
	};

	/**
	 * 概述：校验手机（或账号）和短信验证码是合正确的响应
	 */
	class VerifySmsCodeEvent : public AuthBaseEvent {
	public:

		/** 返回码*/
		int errCode;
		/** 返回码描述*/
		std::string description;
		/** 
		 * 包括如下这些：
		 * <li>{@link UIAction#SUCCESS}</li>
		 * <li>{@link UIAction#FAILED}</li>
		 */
		int uiAction;
		/** 上下文。请求和响应的上下文是呼应的。请求的上下文是什么，这里就是什么*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		VerifySmsCodeEvent():errCode(0),description(""),context(""),uiAction(0)
		{
			OP_CMD=EventOpCmd::OP_VERIFY_SMSCODE;
		}

		VerifySmsCodeEvent(const AuthBaseEvent* base):errCode(0),description(""),context(""),uiAction(0),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_VERIFY_SMSCODE;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errCode);

			char struiAction[16] = {0};
			sprintf(struiAction, "%d", uiAction);

		    mJsonObject["errcode"] = CSJson::Value(strerrcode);
            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["uiaction"] = CSJson::Value(struiAction);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["verify_smscode_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(std::string jsonstr) {
			
			CSJson::Value parent = getParentJsonObject();

			if (!parent.isMember("verify_smscode_res")) {

				errCode = AuthError::Json_Error().code();
				description = AuthError::Json_Error().description();
				uiAction = UIAction::FAILED;
				context = "";
				return;
			}
			CSJson::Value son = parent["verify_smscode_res"];
			std::string strData;
			strData = son["errcode"].asString();
			errCode = atoi(strData.c_str());
			description = son["description"].asString();
			strData = son["uiaction"].asString();
			uiAction = atoi(strData.c_str());
			context = son["context"].asString();
		}
	};

	/**
	 * 概述：注册新用户的响应
	 */
	class RegisterEvent : public AuthBaseEvent {
	public:

		/** 返回码*/
		int errCode;
		/** 返回码描述*/
		std::string description;
		/**
		 * 包括如下这些：
		 * <li>{@link UIAction#SUCCESS}</li>
		 * <li>{@link UIAction#FAILED}</li>
		 */
		int uiAction;
		/** uid */
		std::string uid;
		/** yyid */
		std::string yyid;
		/** uid */
		std::string passport;
		/** 上下文。请求和响应的上下文是呼应的。请求的上下文是什么，这里就是什么*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		RegisterEvent():errCode(0),description(""),context(""),uiAction(0)
		{
			OP_CMD=EventOpCmd::OP_REGISTER;
		}

		RegisterEvent(const AuthBaseEvent* base):errCode(0),description(""),context(""),uiAction(0),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_REGISTER;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errCode);

			char struiAction[16] = {0};
			sprintf(struiAction, "%d", uiAction);

		    mJsonObject["errcode"] = CSJson::Value(strerrcode);
            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["uiaction"] = CSJson::Value(struiAction);
			mJsonObject["uid"] = CSJson::Value(uid);
			mJsonObject["yyid"] = CSJson::Value(yyid);
			mJsonObject["passport"] = CSJson::Value(passport);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["register_res"] = mJsonObject;

			return AuthBaseEvent::marshall();
		}

		void unmarshall(std::string jsonstr) {

			CSJson::Value parent = getParentJsonObject();

			if (!parent.isMember("register_res")) {

				errCode = AuthError::Json_Error().code();
				description = AuthError::Json_Error().description();
				uiAction = UIAction::FAILED;
				context = "";
				return;
			}
			CSJson::Value son = parent["register_res"];
			std::string strData;
			strData = son["errcode"].asString();
			errCode = atoi(strData.c_str());
			description = son["description"].asString();
			strData = son["uiaction"].asString();
			uiAction = atoi(strData.c_str());
			context = son["context"].asString();
			uid = son["uid"].asString();
			yyid = son["yyid"].asString();
			passport = son["passport"].asString();
		}
	};

	/**
	 * 概述：刷新验证码图片回调事件
	 */
	class RefreshPicEvent : public AuthBaseEvent {
	public:

		/** 返回码*/
		int errCode;
		/** 返回码描述*/
		std::string description;
		/** 经过BASE64编码后的验证码图片*/
		std::string pic;
		/** 
		 * 包括如下这些：
		 * <li>{@link UIAction#SUCCESS}</li>
		 * <li>{@link UIAction#FAILED}</li>
		 */
		int uiAction;
		/** 上下文。请求和响应的上下文是呼应的。请求的上下文是什么，这里就是什么*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		RefreshPicEvent():errCode(0),description(""),pic(""),context(""),uiAction(0)
		{
			OP_CMD=EventOpCmd::OP_REFRESH_PIC_EVENT;
		}

		RefreshPicEvent(const AuthBaseEvent* base):errCode(0),description(""),pic(""),context(""),uiAction(0),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_REFRESH_PIC_EVENT;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errCode);

			char struiAction[16] = {0};
			sprintf(struiAction, "%d", uiAction);

		    mJsonObject["errcode"] = CSJson::Value(strerrcode);
            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["uiaction"] = CSJson::Value(struiAction);
			mJsonObject["pic"] = CSJson::Value(pic);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["refreshpic_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(std::string jsonstr) {
		

			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("refreshpic_res")) {
				errCode = AuthError::Json_Error().code();
				description = AuthError::Json_Error().description();
				uiAction = UIAction::FAILED;
				pic = "";
				context = "";
				return;
			}
			CSJson::Value son = parent["refreshpic_res"];

			std::string strData;
			strData = son["errcode"].asString();
			errCode = atoi(strData.c_str());
			description = son["description"].asString();
			strData = son["uiaction"].asString();
			uiAction = atoi(strData.c_str());
			pic = son["pic"].asString();
			context = son["context"].asString();
		}
	};

	/**
	 * 概述：查询请求回调事件
	 */
	class QueryEvent: public AuthBaseEvent {
	public:
		/** 返回码*/
		int errcode;
		/** 错误信息*/
		std::string errmsg;
		/** token*/
		std::string token;
		/** otp*/
		std::string otp;
		/** appid*/
		std::string appid;
		/** 上下文。请求和响应的上下文是呼应的。请求的上下文是什么，这里就是什么*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		QueryEvent() : errcode(0) {
			OP_CMD = EventOpCmd::OP_QUERY_EVENT;
		}

		QueryEvent(const AuthBaseEvent* base):errcode(0),errmsg(""),token(""),context(""),otp(""),appid(""),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_QUERY_EVENT;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

			char strerrcode[16] = { 0 };
			sprintf(strerrcode, "%d", errcode);

			mJsonObject["errcode"] = CSJson::Value(strerrcode);
			mJsonObject["errmsg"] = CSJson::Value(errmsg);
			mJsonObject["token"] = CSJson::Value(token);
			mJsonObject["otp"] = CSJson::Value(otp);
			mJsonObject["appid"] = CSJson::Value(appid);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["query_res"] = mJsonObject;

			return AuthBaseEvent::marshall();
		}

		void unmarshall(std::string jsonstr) {

			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("query_res")) {
				errcode = AuthError::Json_Error().code();
				errmsg = "错误json";
				return;
			}
			CSJson::Value son = parent["query_res"];

			std::string strData = son["errcode"].asString();
			errcode = atoi(strData.c_str());
			errmsg = son["errmsg"].asString();
			appid = son["appid"].asString();
			token = son["token"].asString();
			otp = son["otp"].asString();
			context = son["context"].asString();
		}
	};

	/**
	 * 概述：凭证自动更新
	 */
	class CreditRenewEvent : public AuthBaseEvent {
	public:

		/** UID*/
		std::string uid;
		/** YY号*/
		std::string yyid;
		/** 通行证*/
		std::string passport;
		/** 凭证*/
		std::string credit;

		
		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		
		CreditRenewEvent():uid(""),yyid(""),passport(""),credit("") 
		{
			OP_CMD=EventOpCmd::OP_CREDIT_RENEW;
		}

		CreditRenewEvent(const AuthBaseEvent* base):uid(""),yyid(""),passport(""),credit("") ,AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_CREDIT_RENEW;
		}
		
		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {


		    mJsonObject["uid"] = CSJson::Value(uid);
            mJsonObject["yyid"] = CSJson::Value(yyid);
			mJsonObject["passport"] = CSJson::Value(passport);
			mJsonObject["credit"] = CSJson::Value(credit);

			mRootJsonObject["credit_renew"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(std::string jsonstr) {
			
			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("credit_renew")) {
				uid = "";
				yyid = "";
				passport = "";
				credit = "";
				return;
			}
			CSJson::Value son = parent["credit_renew"];

			uid = son["uid"].asString();
			yyid = son["yyid"].asString();
			passport = son["passport"].asString();
			credit = son["credit"].asString();
		}
	};

	/**
	 * 概述：匿名回调事件（业务端不需要关注该事件）
	 */
	class AnonymousEvent : public AuthBaseEvent {
	public:

		/** 返回码*/
		int errCode;
		/** 返回码描述*/
		std::string description;
		/** UID*/
		std::string uid;
		/** YY号*/
		std::string yyid;
		/** 通行证*/
		std::string passport;
		/** 
		 * 包括如下这些：
		 * <li>{@link UIAction#SUCCESS}</li>
		 * <li>{@link UIAction#FAILED}</li>
		 */
		int uiAction;
		/*
		*分配的密码
		*/
		std::string password;

		
		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		AnonymousEvent():errCode(0),description(""),uid(""),yyid(""),passport(""),password(""),uiAction(0) 
		{
			OP_CMD=EventOpCmd::OP_ANONYMOUS_EVENT;
		}

		AnonymousEvent(const AuthBaseEvent* base):errCode(0),description(""),uid(""),yyid(""),passport(""),password(""),uiAction(0)  ,AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_ANONYMOUS_EVENT;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errCode);

			char struiAction[16] = {0};
			sprintf(struiAction, "%d", uiAction);

		    mJsonObject["errcode"] = CSJson::Value(strerrcode);
            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["uiaction"] = CSJson::Value(struiAction);
			mJsonObject["uid"] = CSJson::Value(uid);
			mJsonObject["yyid"] = CSJson::Value(yyid);
			mJsonObject["passport"] = CSJson::Value(passport);
			mJsonObject["password"] = CSJson::Value(password);

			mRootJsonObject["anonymous_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(std::string jsonstr) {
		

			CSJson::Value parent = getParentJsonObject();

			if (!parent.isMember("anonymous_res")) {
                errCode = AuthError::Json_Error().code();
				description = AuthError::Json_Error().description();
				uid = "";
				yyid = "";
				passport = "";
				return;
			}
			CSJson::Value son = parent["anonymous_res"];
			std::string strData;
			strData = son["errcode"].asString();
			errCode = atoi(strData.c_str());
			description = son["description"].asString();
			strData = son["uiaction"].asString();
			uiAction = atoi(strData.c_str());
			uid = son["uid"].asString();
			yyid = son["yyid"].asString();
			passport = son["passport"].asString();
			password = son["password"].asString();
		}
	};

	/**
	 * 概述：请求超时
	 */
	class TimeoutEvent : public AuthBaseEvent {
	public:

		/** 上下文。请求和响应的上下文是呼应的。请求的上下文是什么，这里就是什么*/
		std::string context;
		std::string descritption;
		std::string detail;
		int uiaction;
		int op_cmd;

		
		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		TimeoutEvent():context(""), descritption(""), detail(""), uiaction(0), op_cmd(0)
		{
			OP_CMD=EventOpCmd::OP_TIME_OUT;
		}

		TimeoutEvent(const AuthBaseEvent* base):context(""), descritption(""), detail(""), uiaction(0), op_cmd(0), AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_TIME_OUT;
		}

		string marshall() {

			mJsonObject["context"] = CSJson::Value(context);
			mJsonObject["descritption"] = CSJson::Value(descritption);
			mJsonObject["detail"] = CSJson::Value(detail);
			char strop_cmd[16] = {0};
			sprintf(strop_cmd, "%d", op_cmd);
			mJsonObject["op_cmd"] = CSJson::Value(strop_cmd);

			char struiAction[16] = {0};
			sprintf(struiAction, "%d", uiaction);
			mJsonObject["uiaction"] = CSJson::Value(struiAction);

			mRootJsonObject["timeout"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(std::string jsonstr) {
		
			CSJson::Value parent = getParentJsonObject();

			if (!parent.isMember("timeout")) {
				context = "";
				descritption = "";
				detail = "";
				op_cmd = 0;
				uiaction = 0;
				return;
			}
			CSJson::Value son = parent["timeout"];
			std::string strData;
			context = son["context"].asString();
			descritption = son["description"].asString();
			detail = son["detail"].asString();
			strData = son["uiaction"].asString();
			uiaction = atoi(strData.c_str());
			strData = son["op_cmd"].asString();
			op_cmd = atoi(strData.c_str());	
		}
	};

	/**
	 * 概述：改密检测结果返回
	 */
	class CheckModPwdEvent : public AuthBaseEvent {
	public:
		/*取值参考UiAction枚举*/
		int uiaction;
		/*后台返回码*/
		int errcode;
		/*由服务器端返回的提示信息*/
		std::string description;
		/*手机号掩码*/
		std::string mobile_mask;
		/*邮箱掩码*/
		std::string email_mask;
		/*跳转到web改密的url*/
		std::string url;
		/*1：是登录手机，0：不是*/
		int is_login_mobile;
	    /** 上下文*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		CheckModPwdEvent():uiaction(0),errcode(0),description(""),mobile_mask(""),email_mask(""),url(""),is_login_mobile(0),context("")
		{
			OP_CMD=EventOpCmd::OP_CHECK_MODPWD;
		}

		CheckModPwdEvent(const AuthBaseEvent* base):uiaction(0),errcode(0),description(""),mobile_mask(""),email_mask(""),url(""),is_login_mobile(0),context(""),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_CHECK_MODPWD;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {
			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errcode);

			char struication[16] = {0};
			sprintf(struication, "%d", uiaction);

			char strisloginmobile[16] = {0};
			sprintf(strisloginmobile, "%d", is_login_mobile);

			mJsonObject["errcode"] = CSJson::Value(strerrcode);
			mJsonObject["uiaction"] = CSJson::Value(struication);
			mJsonObject["is_login_mobile"] = CSJson::Value(strisloginmobile);

            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["mobile_mask"] = CSJson::Value(mobile_mask);
            mJsonObject["email_mask"] = CSJson::Value(email_mask);
			mJsonObject["url"] = CSJson::Value(url);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["check_modpwd_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(string jsonstr) {
			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("check_modpwd_res")) {
				description = "";
				mobile_mask = "";
				email_mask = "";
				url = "";
				context = "";
				uiaction = 0;
				errcode = 0;
				is_login_mobile = 0;
				return;
			}
			std::string strData;
			CSJson::Value son = parent["check_modpwd_res"];
			description = son["description"].asString(); 
			mobile_mask = son["mobile_mask"].asString();  
			email_mask = son["email_mask"].asString();   
			url = son["url"].asString();  
			context = son["context"].asString();

			strData = son["uiaction"].asString();
			uiaction = atoi(strData.c_str());

			strData = son["errcode"].asString();
			errcode = atoi(strData.c_str());

			strData = son["is_login_mobile"].asString();
			is_login_mobile = atoi(strData.c_str());
		}
	};

	/**
	 * 概述：改密结果返回
	 */
	class ModPwdEvent : public AuthBaseEvent {
	public:
		/*取值参考UiAction枚举*/
		int uiaction;
		/*后台返回码*/
		int errcode;
		/*由服务器端返回的提示信息*/
		std::string description;
		/*手机号掩码*/
		std::string mobile_mask;
		/*邮箱掩码*/
		std::string email_mask;
		/*uid*/
		std::string uid;
		/*YY号*/
		std::string yyid;
		/*YY通行证*/
		std::string passport;
		/*登陆成功时返回的凭证*/
		std::string credit;
	    /** 上下文*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		ModPwdEvent():uiaction(0),errcode(0),description(""),mobile_mask(""),email_mask(""),uid(""),yyid(""),passport(""),credit(""),context("")
		{
			OP_CMD=EventOpCmd::OP_MODPWD;
		}

		ModPwdEvent(const AuthBaseEvent* base):uiaction(0),errcode(0),description(""),mobile_mask(""),email_mask(""),uid(""),yyid(""),passport(""),credit(""),context(""),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_MODPWD;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {
			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errcode);

			char struication[16] = {0};
			sprintf(struication, "%d", uiaction);

			mJsonObject["errcode"] = CSJson::Value(strerrcode);
			mJsonObject["uiaction"] = CSJson::Value(struication);

            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["mobile_mask"] = CSJson::Value(mobile_mask);
            mJsonObject["email_mask"] = CSJson::Value(email_mask);
			mJsonObject["uid"] = CSJson::Value(uid);
			mJsonObject["yyid"] = CSJson::Value(yyid);
			mJsonObject["passport"] = CSJson::Value(passport);
			mJsonObject["credit"] = CSJson::Value(credit);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["modpwd_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(string jsonstr) {
			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("modpwd_res")) {
				description = "";
				mobile_mask = "";
				email_mask = "";
				uid = "";
				yyid = "";
				passport = "";
				credit = "";
				context = "";
				uiaction = 0;
				errcode = 0;
				return;
			}
			std::string strData;
			CSJson::Value son = parent["modpwd_res"];
			description = son["description"].asString(); 
			mobile_mask = son["mobile_mask"].asString();  
			email_mask = son["email_mask"].asString();   
			uid = son["uid"].asString();  
			yyid = son["yyid"].asString();  
			passport = son["passport"].asString();  
			credit = son["credit"].asString();  
			context = son["context"].asString();

			strData = son["uiaction"].asString();
			uiaction = atoi(strData.c_str());

			strData = son["errcode"].asString();
			errcode = atoi(strData.c_str());
		}
	};

	/**
	 * 概述：注册检测结果返回
	 */
	class CheckRegEvent : public AuthBaseEvent {
	public:
		/*取值参考UiAction枚举*/
		int uiaction;
		/*后台返回码*/
		int errcode;
		/*由服务器端返回的提示信息*/
		std::string description;
	    /** 上下文*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		CheckRegEvent():uiaction(0),errcode(0),description(""),context("")
		{
			OP_CMD=EventOpCmd::OP_CHECK_REG;
		}

		CheckRegEvent(const AuthBaseEvent* base):uiaction(0),errcode(0),description(""),context(""),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_CHECK_REG;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {
			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errcode);

			char struication[16] = {0};
			sprintf(struication, "%d", uiaction);

			mJsonObject["errcode"] = CSJson::Value(strerrcode);
			mJsonObject["uiaction"] = CSJson::Value(struication);
            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["check_reg_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(string jsonstr) {
			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("check_reg_res")) {
				description = "";
				context = "";
				uiaction = 0;
				errcode = 0;
				return;
			}
			std::string strData;
			CSJson::Value son = parent["check_reg_res"];
			description = son["description"].asString(); 
			context = son["context"].asString();

			strData = son["uiaction"].asString();
			uiaction = atoi(strData.c_str());

			strData = son["errcode"].asString();
			errcode = atoi(strData.c_str());
		}
	};

	/**
	 * 概述：用户检测结果返回
	 */
	class CheckUserEvent : public AuthBaseEvent {
	public:
		/*取值参考UiAction枚举*/
		int uiaction;
		/*后台返回码*/
		int errcode;
		/*由服务器端返回的提示信息*/
		std::string description;
	    /** 上下文*/
		std::string context;
		/*1:存在此用户,0:不存在此用户, 999:未知的初始值*/
		int is_exist;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		CheckUserEvent():uiaction(0),errcode(0),description(""),context(""),is_exist(999)
		{
			OP_CMD=EventOpCmd::OP_CHECK_USER;
		}

		CheckUserEvent(const AuthBaseEvent* base):uiaction(0),errcode(0),description(""),context(""),is_exist(999),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_CHECK_USER;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {
			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errcode);

			char struication[16] = {0};
			sprintf(struication, "%d", uiaction);

			mJsonObject["errcode"] = CSJson::Value(strerrcode);
			mJsonObject["uiaction"] = CSJson::Value(struication);
            mJsonObject["description"] = CSJson::Value(description);
			mJsonObject["context"] = CSJson::Value(context);
			mJsonObject["is_exist"] = CSJson::Value(is_exist);

			mRootJsonObject["check_user_res"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(string jsonstr) {
			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("check_user_res")) {
				description = "";
				context = "";
				uiaction = 0;
				errcode = 0;
				return;
			}
			std::string strData;
			CSJson::Value son = parent["check_user_res"];
			description = son["description"].asString(); 
			context = son["context"].asString();

			strData = son["uiaction"].asString();
			uiaction = atoi(strData.c_str());

			strData = son["errcode"].asString();
			errcode = atoi(strData.c_str());

			strData = son["is_exist"].asString();
			is_exist = atoi(strData.c_str());
		}
	};

	class Exchange3rdTokenEvent : public AuthBaseEvent {
	public:
		/*上下文*/
		std::string  context;
		/**UID*/
		uint64_t     uid;
		/**返回的票据*/
		std::string  accesstoken;
		/**票据有效时间，秒*/
		uint32_t     token_validtime;
		/**错误码*/
		uint32_t     errcode;
		/**错误提示*/
		std::string  errmsg;
		/**保留字段*/
		std::string  reserve;
		CSJson::Value mJsonObject;
		Exchange3rdTokenEvent():context(""),uid(0),accesstoken(""),token_validtime(0),errcode(0),errmsg(""),reserve("")
		{
			OP_CMD = EventOpCmd::OP_CHECK_THIRDTOKEN;
		}
		Exchange3rdTokenEvent(const AuthBaseEvent* base):context(""),uid(0),accesstoken(""),token_validtime(0),errcode(0),errmsg(""),reserve(""),AuthBaseEvent(base)
		{
			OP_CMD = EventOpCmd::OP_CHECK_THIRDTOKEN;
		}
		int opCmd() {
			return AuthBaseEvent::opCmd();
		}
		std::string marshall()
		{
			mJsonObject["context"] = CSJson::Value(context);
			mJsonObject["accesstoken"] = CSJson::Value(accesstoken);
			mJsonObject["errmsg"] = CSJson::Value(errmsg);
			mJsonObject["reserve"] = CSJson::Value(reserve);
			char struid[16] = {0};
			sprintf(struid, "%d", uid);
			mJsonObject["uid"] = CSJson::Value(struid);
			char strerrcode[16] = {0};
			sprintf(strerrcode, "%d", errcode);
			mJsonObject["errcode"] = CSJson::Value(strerrcode);
			char strtime[16] = {0};
			sprintf(strtime, "%d", token_validtime);
			mJsonObject["token_validtime"] = CSJson::Value(strtime);

			mRootJsonObject["exchange_3rdtoken_res"] = mJsonObject;
			return AuthBaseEvent::marshall();
		}

		void unmarshall(string jsonstr) 
		{
			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("exchange_3rdtoken_res")) {
				accesstoken = "";
				errmsg = "";
				reserve = "";
				uid = 0;
				context = "";
				token_validtime = 0;
				errcode = 0;
				return;
			}
			CSJson::Value son = parent["exchange_3rdtoken_res"];
			context = son["context"].asString();
			accesstoken = son["accesstoken"].asString();
			errmsg = son["errmsg"].asString();
			reserve = son["reserve"].asString();
			if (!son["uid"].asString().empty())
			{
				uid = atoi(son["uid"].asString().c_str());
			}
			if (!son["token_validtime"].asString().empty())
			{
				token_validtime = atoi(son["token_validtime"].asString().c_str());
			}
			if (!son["errcode"].asString().empty())
			{
				errcode = atoi(son["errcode"].asString().c_str());
			}
		}
	};

	class QuickGetYYTicketEvent : public AuthBaseEvent {
	public:
		/** UID*/
		std::string uid;
		/** YYID*/
		std::string yyid;
		/** 昵称*/
		std::string nick;
		/** 票据*/
		std::string ticket;
	    /** 上下文*/
		std::string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		QuickGetYYTicketEvent():uid(""),yyid(""),nick(""),ticket(""),context("")
		{
			OP_CMD=EventOpCmd::OP_GETQUICK_YYTICKET;
		}

		QuickGetYYTicketEvent(const AuthBaseEvent* base):uid(""),yyid(""),nick(""),ticket(""),context(""),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_GETQUICK_YYTICKET;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

            mJsonObject["uid"] = CSJson::Value(uid);
			mJsonObject["yyid"] = CSJson::Value(yyid);
            mJsonObject["nick"] = CSJson::Value(nick);
			mJsonObject["ticket"] = CSJson::Value(ticket);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["get_yy_ticket"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(string jsonstr) {
			CSJson::Value parent = getParentJsonObject();
			if (!parent.isMember("get_yy_ticket")) {
				uid = "";
				yyid = "";
				nick = "";
				ticket = "";
				context = "";
				return;
			}
			CSJson::Value son = parent["get_yy_ticket"];
			uid = son["uid"].asString(); 
			yyid = son["yyid"].asString();  
			nick = son["nick"].asString();   
			ticket = son["ticket"].asString();  
			context = son["context"].asString(); 
		}
	};

	class YYUserInfo  {
	public:
		
		std::string imgpath;// STRATEGY::NONE  // TODO
		/** 选择界面显示的标题，如：短信验证码、图片验证码……*/
		std::string yyid ;
		/** 界面显示的标题，如：你绑定的手机号是*/
		std::string nick ;
		std::string uid ;
		std::string passport;
		std::string ver;
		int handler;

		YYUserInfo(): imgpath(""),yyid(""),nick(""),uid(""),ver(""),passport(""),handler(0){
		}
	};

	class QuickGetYYListEvent : public AuthBaseEvent {

	public:
		std::string context;
		std::vector<YYUserInfo> YYUserInfos ;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		QuickGetYYListEvent():context("")
		{
			OP_CMD=EventOpCmd::OP_GETQUICK_YYLIST;
		}

		QuickGetYYListEvent(const AuthBaseEvent* base):context(""),AuthBaseEvent(base)
		{
			OP_CMD=EventOpCmd::OP_GETQUICK_YYLIST;
		}

		int opCmd() {
			return AuthBaseEvent::opCmd();
		}

		string marshall() {

			
			mJsonObject["context"] = CSJson::Value(context);

            CSJson::Value arr_value(CSJson::arrayValue);
			for(int i = 0; i < YYUserInfos.size(); ++i)
			{
				char strhandler[16] = {0};
			    itoa(YYUserInfos[i].handler, strhandler, 10);

				CSJson::Value json_nv;
				json_nv["imgpath"] = CSJson::Value(YYUserInfos[i].imgpath);
				json_nv["yyid"] = CSJson::Value(YYUserInfos[i].yyid);
				json_nv["nick"] = CSJson::Value(YYUserInfos[i].nick);
				json_nv["uid"] = CSJson::Value(YYUserInfos[i].uid);
				json_nv["passport"] = CSJson::Value(YYUserInfos[i].passport);
				json_nv["ver"] = CSJson::Value(YYUserInfos[i].ver);
				json_nv["handler"] = CSJson::Value(strhandler);
				arr_value[i] = CSJson::Value(json_nv);
			}

            mJsonObject["UerInfo"]=arr_value;
			mRootJsonObject["get_yy_list"] = mJsonObject;
         
			return AuthBaseEvent::marshall();
		}

		void unmarshall(string jsonstr) {
			CSJson::Value parent = getParentJsonObject();

			if (!parent.isMember("get_yy_list")) {
				context = "";
				return;
			}
			std::string strData;
			CSJson::Value son = parent["get_yy_list"];

			context = son["context"].asString();   
			YYUserInfos.clear();

			if (son.isMember("UerInfo") ) {

				CSJson::Value uYYUserInfos = son["UerInfo"];

				for (CSJson::Value::iterator itr = uYYUserInfos.begin(); itr != uYYUserInfos.end(); ++itr) {
	                CSJson::Value &it_info = *itr;

                    YYUserInfo info ;
					info.imgpath = it_info["imgpath"].asString();
					info.yyid = it_info["yyid"].asString(); 
					info.nick = it_info["nick"].asString();  
					info.uid = it_info["uid"].asString();   
					info.passport = it_info["passport"].asString(); 
					info.ver =  it_info["ver"].asString();  
					std::string strhandler = it_info["handler"].asString();
					info.handler = atoi(strhandler.c_str());
					YYUserInfos.push_back(info);
				}
			}
		}
	};



	class AuthEventConverter
	{
	public:
		/**
		 * 根据Json字符串生成一个回调事件对象
		 * @param authData	Json字符串
		 * @return	如果失败则返回空
		 */
		// TODO 这里动态配了内存，要考虑清理动作；
		static AuthBaseEvent* toAuthEvent(std::string authData) {
			AuthBaseEvent base ;
			base.unmarshall(authData);

			switch (base.opCmd()) {
				case EventOpCmd::OP_LOGIN_EVENT : {
					//回调事件类型：登录事件
					LoginEvent* et = new LoginEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_SEND_SMS_EVENT: {
					//回调事件类型：下发短信
					SendSmsEvent* et = new SendSmsEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_REFRESH_PIC_EVENT: {
					//回调事件类型：刷新图片验证码
					RefreshPicEvent* et = new RefreshPicEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_QUERY_EVENT: {
					//回调事件类型：查询请求
					QueryEvent* et = new QueryEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_ANONYMOUS_EVENT: {
					//回调事件类型：匿名登录
					AnonymousEvent* et = new AnonymousEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_TIME_OUT: {
					//回调事件类型：操作超时
					TimeoutEvent* et = new TimeoutEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_CREDIT_RENEW: {
					//回调事件类型：自动刷新凭证
					CreditRenewEvent* et = new CreditRenewEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_VERIFY_SMSCODE: {
					//回调事件类型：检验短信验证码
					VerifySmsCodeEvent* et = new VerifySmsCodeEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_REGISTER: {
					//回调事件类型：
					RegisterEvent* et = new RegisterEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_MODPWD: {
					//回调事件类型：
					ModPwdEvent* et = new ModPwdEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_CHECK_MODPWD: {
					//回调事件类型：
					CheckModPwdEvent* et = new CheckModPwdEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_CHECK_REG: 
				{
					CheckRegEvent* et = new CheckRegEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_CHECK_USER: 
				{
					CheckUserEvent* et = new CheckUserEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_CHECK_THIRDTOKEN:
				{
					Exchange3rdTokenEvent* et = new Exchange3rdTokenEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				case EventOpCmd::OP_GETQUICK_YYTICKET:
				{
					QuickGetYYTicketEvent* et = new QuickGetYYTicketEvent(&base);
					et->unmarshall(authData);
					return et;
				}
			case EventOpCmd::OP_GETQUICK_YYLIST:
				{
					QuickGetYYListEvent* et = new QuickGetYYListEvent(&base);
					et->unmarshall(authData);
					return et;
				}
				default: {
					return NULL;
				}
			}
		}
	};

}


#endif
