﻿#ifndef  __UDBAUTH_AUTHREQUEST___H__
#define __UDBAUTH_AUTHREQUEST___H__

#include <string>
#include <sstream>
#include <iomanip>
#include <stdlib.h>


#include "json_lib.h"
using namespace std;


namespace udbauth
{

	/** 
	 * 概述：请求类型
	 */
    class RequestOpCmd {

	public:
		/** 请求类型：非法请求*/
		const static int OP_INVALID_REQ = -1;
		/** 请求类型：密码登录*/
		const static int OP_PASSWORD_LOGIN = 1;
		/** 请求类型：凭证登录*/
		const static int OP_CREDIT_LOGIN = 2;
		/** 请求类型：票据登录*/
		const static int OP_TICKET_LOGIN = 3;
		/** 请求类型：下发短信*/
		const static int OP_SEND_SMS = 4;
		/** 请求类型：短信注册登录*/
		const static int OP_SMS_LOGIN = 5;
		/** 请求类型：短信改密*/
		const static int OP_SMS_MOD_PWD = 6;
		/** 请求类型：刷新图片*/
		const static int OP_REFRESH_PIC = 7;
		/** 请求类型：查询*/
		const static int OP_QUERY = 8;
		/** 请求类型：快速改密（用于下发短信之后进行改密）*/
		const static int OP_QUICK_MOD_PWD = 9;
		/** 请求类型：上报日志*/
		const static int OP_LOG_REPORT = 10;
		/** 请求类型：检查用户是否发送短信到后端了*/
		const static int OP_CHECK_SMS_UP = 11;
		/** 请求类型：检验手机（或账号）和短信验证码*/
		const static int OP_VERIFY_SMSCODE = 12;
		/** 请求类型：第三方登录*/
		const static int OP_THIRD_LOGIN = 13;
		/** 请求类型：二维码登录*/
		const static int OP_QRCODE_LOGIN = 14;
		/** 请求类型：取消请求*/
		const static int OP_CANCEL = 15;
		/** 请求类型：改密检测请求*/
		const static int OP_CHECK_MODPWD = 16;
		/** 请求类型：注册检测请求*/
		const static int OP_CHECK_REG = 17;
		/** 请求类型：用户检测请求*/
		const static int OP_CHECK_USER = 18;
		/** 请求类型：交换第三方票据*/
		const static int OP_CHECK_THIRDTOKEN = 23;
		/**请求类型：快速登录获取用户信息*/
		const static int OP_GETQUICK_YYLIST = 100;
		/**请求类型：快速登录获取Ticket*/
		const static int OP_GETQUICK_TICKET = 101;
	};

	/**
	 * 概述：二次验证策略
	 */
	class STRATEGY {
	public:
		/** 二次验证策略：无 */
		const static  int NONE = 0x0;
		/** 二次验证策略：图片验证码 */
		const static  int PICCODE = 0x01;
		/** 二次验证策略：手机令牌 */
		const static  int MOBTOKEN = 0x02;
		/** 二次验证策略：硬件令牌 */
		const static  int HWTOKEN = 0x04;
		/** 二次验证策略：短信验证码 */
		const static  int SMSCODE = 0x08;
		/** 二次验证策略：滑动验证码（Web交互验证） */
		const static  int SLIDE = 0x10;
		/** 二次验证策略：短信上行（用户手动发送短信到指定端口，然后调用{@link CheckSmsUpReq}发送请求，检查是否发送短信成功） */
		const static  int SMS_UP = 0x20;
	};

	
	/**
	 * 概述：登录请求的基础类
	 */
	class AuthBaseReq {
	public:

		/** 请求的类型, 参考 {@link RequestOpCmd}*/
		int OP_CMD ;

		/** 业务JSON内容*/
	    string json ;

		/** JSON版本号*/
		int json_ver;

		///** 最外层JSON*/
		CSJson::Value mRootJsonObject;

		AuthBaseReq():OP_CMD(RequestOpCmd::OP_INVALID_REQ),json_ver(0){
		}
		AuthBaseReq(const AuthBaseReq* base){
			this->json = base->json;
			this->OP_CMD = base->OP_CMD;
			this->json_ver = base->json_ver;
			this->mRootJsonObject = base->mRootJsonObject;
		}

		///** 将数据转化为Json，再转为string*/
		string marshall() {

			char strcmd[10] = {0};
            sprintf(strcmd, "%d", OP_CMD);
            char strjver[10] = {0};
            sprintf(strjver, "%d", json_ver);

            mRootJsonObject["op_cmd"] = CSJson::Value(strcmd);
            mRootJsonObject["json_ver"] = CSJson::Value(strjver);

			CSJson::FastWriter fast_writer;
			json=fast_writer.write(mRootJsonObject);
            return json;
		}

		void unmarshall(const string& jsonstr) {
			try {
				json = jsonstr;

				CSJson::Reader jreader;
				if (jreader.parse(jsonstr.c_str(), mRootJsonObject))
				{
					if(mRootJsonObject.isMember("op_cmd") ){
						string str_op_cmd = mRootJsonObject["op_cmd"].asString();
						OP_CMD = atoi(str_op_cmd.c_str());
					}

					if(mRootJsonObject.isMember("json_ver") ){
						string str_json_ver = mRootJsonObject["json_ver"].asString();
						json_ver = atoi(str_json_ver.c_str());
					}
				}
			} catch (std::runtime_error e) {
				OP_CMD = RequestOpCmd::OP_INVALID_REQ;
			}
		}

	};

	/**
	 * 概述：“账号+密码”登录
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class LoginReq : public AuthBaseReq {
	public:

		/** 通行证，YY号，邮箱，手机号之一*/
	    string user;
		/** 密码，SHA1加密*/
		string password;
		/** 二次验证策略类型，见{@link STRATEGY}类声明常量 */
	    int strategy ;
		/** 二次验证码，如：图片验证就是图片验证码、短信验证就是验证验证码、手机令牌和硬件令牌类似*/
		string stra_token;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject;

		LoginReq():user(""),password(""),strategy(STRATEGY::NONE),stra_token(""),context("")
		{
            OP_CMD = RequestOpCmd::OP_PASSWORD_LOGIN;
		}

		LoginReq(const AuthBaseReq* base):user(""),password(""),strategy(STRATEGY::NONE),stra_token(""),context(""),AuthBaseReq(base)
		{
			OP_CMD = RequestOpCmd::OP_PASSWORD_LOGIN;
		}

		/**
		 * 构造一个“账号+密码”登录请求
		 * @param user		通行证，YY号，邮箱，手机号之一
		 * @param password	密码，SHA1加密
		 * @param strategy	用户所选择的二次验证策略的类型
		 * @param stra_token	二次验证码，如：短信验证就是验证验证码、手机令牌和硬件令牌类似。当为STRATEGY::NONE，这参数将被忽略
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		LoginReq(string user, string password, int strategy, string stra_token, string context) {
			OP_CMD = RequestOpCmd::OP_PASSWORD_LOGIN;
			this->user = user;
			this->password = password;
			this->strategy = strategy;
			this->stra_token = stra_token;
			this->context = context;
		}

		string marshall() {

            char strstrategy[16] = {0};
            sprintf(strstrategy, "%d", strategy);

		    mJsonObject["user"] = CSJson::Value(user);
            mJsonObject["password"] = CSJson::Value(password);
			mJsonObject["strategy"] = CSJson::Value(strstrategy);
            mJsonObject["stra_token"] = CSJson::Value(stra_token);
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["pwd_login"] = mJsonObject;
         
			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {


    		if(!mRootJsonObject.isMember("pwd_login")){
    			return;
    		}
    		mJsonObject = mRootJsonObject["pwd_login"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

    		if(!mJsonObject.isMember("user") || mJsonObject["user"].asString().empty()){
    			return;
    		}
    		user = mJsonObject["user"].asString();

			if(mJsonObject.isMember("password") && !mJsonObject["password"].asString().empty()){
				password = mJsonObject["password"].asString();
			}

			if(mJsonObject.isMember("strategy") && !mJsonObject["strategy"].asString().empty()){
				strategy = atoi(mJsonObject["strategy"].asString().c_str());
			}

			if(mJsonObject.isMember("stra_token") && !mJsonObject["stra_token"].asString().empty()){
				stra_token = mJsonObject["stra_token"].asString();
			}

		}
	};


	/**
	 * 概述：凭证登录请求
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class CreditLoginReq : public AuthBaseReq {
	public:

		/** 用户唯一标识UID*/
		unsigned long long uid;
		/** 登录凭证*/
		string credit;
		/** 二次验证策略类型，见{@link STRATEGY}类声明常量 */
		int strategy ;
		/** 二次验证码，如：短信验证就是验证验证码、手机令牌和硬件令牌类似*/
		string stra_token;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		CreditLoginReq():uid(0),credit(""),strategy(STRATEGY::NONE),stra_token(""),context("")
		{
			OP_CMD = RequestOpCmd::OP_CREDIT_LOGIN;
		}

		CreditLoginReq(const AuthBaseReq* base):uid(0),credit(""),strategy(STRATEGY::NONE),stra_token(""),context(""),AuthBaseReq(base)
		{
			OP_CMD = RequestOpCmd::OP_CREDIT_LOGIN;
		}

		/**
		 * 构建一个凭证登录请求
		 * @param uid	UID
		 * @param credit	登录凭证（上次登录返回的凭证）
		 * @param strategy	用户所选择的二次验证策略的类型
		 * @param stra_token	二次验证码，如：短信验证就是验证验证码、手机令牌和硬件令牌类似。当为STRATEGY::NONE，这参数将被忽略
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
	    CreditLoginReq(unsigned long long uid, string credit, int strategy, string stra_token, string context) {
	    	OP_CMD = RequestOpCmd::OP_CREDIT_LOGIN;
			this->uid = uid;
			this->credit = credit;
			this->strategy = strategy;
			this->stra_token = stra_token;
			this->context = context;
		}
		
		string marshall() {

			ostringstream oss;
			oss<<uid;
			std::string struid=oss.str();

            char strstrategy[16] = {0};
            sprintf(strstrategy, "%d", strategy);

		    mJsonObject["uid"] = CSJson::Value(struid);
            mJsonObject["credit"] = CSJson::Value(credit);
			mJsonObject["strategy"] = CSJson::Value(strstrategy);
            mJsonObject["stra_token"] = CSJson::Value(stra_token);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["credit_login"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("credit_login")){
				return;
			}
			mJsonObject = mRootJsonObject["credit_login"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if( mJsonObject.isMember("uid") && !mJsonObject["uid"].asString().empty()){
				//uid = atoll(mJsonObject["uid"].asString().c_str());

				istringstream iss(mJsonObject["uid"].asString().c_str());
	            iss >> std::setbase(10) >> uid;
				
			}

			if ( mJsonObject.isMember("credit") && !mJsonObject["credit"].asString().empty()){
				credit = mJsonObject["credit"].asString();
			}

			if(mJsonObject.isMember("strategy") && !mJsonObject["strategy"].asString().empty()){
				strategy = atoi(mJsonObject["strategy"].asString().c_str());
			}

			if(mJsonObject.isMember("stra_token") && !mJsonObject["stra_token"].asString().empty()){
				stra_token = mJsonObject["stra_token"].asString();
			}

		}
	};

	/**
	 * 概述：票据登陆请求，某些兼容旧版本接口，建议统一使用凭证登陆。
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class TicketLoginReq : public AuthBaseReq {
	public:
		/** 用户唯一标识UID*/
		unsigned long long uid;
		/** 登录票据*/
		string ticket;
		/** 生成登录的appid*/
		string appid;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		TicketLoginReq():uid(0),ticket(""),appid(""),context("")
		{
			OP_CMD = RequestOpCmd::OP_TICKET_LOGIN;
		}

		TicketLoginReq(const AuthBaseReq* base):uid(0),ticket(""),appid(""),context(""),AuthBaseReq(base)
		{
			OP_CMD = RequestOpCmd::OP_TICKET_LOGIN;
		}

		/**
		 * 构造一个票据登录的请求
		 * @param uid	用户唯一标识UID
		 * @param ticket	登录票据
		 * @param appid	生成票据的appid
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		TicketLoginReq(unsigned long long uid, string ticket, string appid, string context) {
			OP_CMD = RequestOpCmd::OP_TICKET_LOGIN;
			this->uid = uid;
			this->ticket = ticket;
			this->appid = appid;
			this->context = context;
		}
		
		string marshall() {
			ostringstream oss;
			oss<<uid;
			std::string struid=oss.str();

			mJsonObject["uid"] = CSJson::Value(struid);
            mJsonObject["ticket"] = CSJson::Value(ticket);
            mJsonObject["appid"] = CSJson::Value(appid);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["ticket_login"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("ticket_login")){
				return;
			}
			mJsonObject = mRootJsonObject["ticket_login"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if( mJsonObject.isMember("uid") && !mJsonObject["uid"].asString().empty()){
				istringstream iss(mJsonObject["uid"].asString().c_str());
	            iss >> std::setbase(10) >> uid;
			}

			if ( mJsonObject.isMember("ticket") && !mJsonObject["ticket"].asString().empty()){
				ticket = mJsonObject["ticket"].asString();
			}

			if ( mJsonObject.isMember("appid") && !mJsonObject["appid"].asString().empty()){
				appid = mJsonObject["appid"].asString();
			}
		}
	};

	
	/**
	 * 概述：二维码登录请求。
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class QrcodeLoginReq : public AuthBaseReq {
	public:

		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;
		/** 登录数据。从二维码HTTP服务获取的login_data，这个login_data本身是个加密内容，使用方获取后直接丢进来就可以*/
		string login_data;
		/** 二次验证策略类型，见{@link STRATEGY}类声明常量 */
		int strategy ;
		/** 二次验证码，如：短信验证就是验证验证码、手机令牌和硬件令牌类似*/
		string stra_token;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		QrcodeLoginReq():context(""),login_data(""),strategy(STRATEGY::NONE),stra_token("")
		{
			OP_CMD = RequestOpCmd::OP_QRCODE_LOGIN;

		}

		QrcodeLoginReq(const AuthBaseReq* base):context(""),login_data(""),strategy(STRATEGY::NONE),stra_token(""),AuthBaseReq(base)
		{
			OP_CMD = RequestOpCmd::OP_QRCODE_LOGIN;

		}

		/**
		 * 构建一个二维码登录请求
		 * @param login_data	登录数据，从二维码HTTP服务获取的login_data，这个login_data本身是个json，使用方获取后直接丢进来就可以
		 * @param strategy	用户所选择的二次验证策略的类型
		 * @param stra_token	二次验证码，如：短信验证就是验证验证码、手机令牌和硬件令牌类似。当为STRATEGY::NONE，这参数将被忽略
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		QrcodeLoginReq(string login_data, int strategy, string stra_token, string context) {
			OP_CMD = RequestOpCmd::OP_QRCODE_LOGIN;
			this->login_data = login_data;
			this->context = context;
			this->strategy = strategy;
			this->stra_token = stra_token;
		}

		string marshall() {
            char strstrategy[16] = {0};
            sprintf(strstrategy, "%d", strategy);
			mJsonObject["context"] = CSJson::Value(context);
			mJsonObject["login_data"] = CSJson::Value(login_data);
			mJsonObject["strategy"] = CSJson::Value(strstrategy);
            mJsonObject["stra_token"] = CSJson::Value(stra_token);
            mRootJsonObject["qrcode_login"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {


			if(!mRootJsonObject.isMember("qrcode_login")){
				return;
			}
			mJsonObject = mRootJsonObject["qrcode_login"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if ( !mJsonObject.isMember("login_data") || mJsonObject["login_data"].asString().empty()){
				return;
			}
			login_data = mJsonObject["login_data"].asString();

			if(mJsonObject.isMember("strategy") && !mJsonObject["strategy"].asString().empty()){
				strategy = atoi(mJsonObject["strategy"].asString().c_str());
			}

			if(mJsonObject.isMember("stra_token") && !mJsonObject["stra_token"].asString().empty()){
				stra_token = mJsonObject["stra_token"].asString();
			}

		}
	};

	/**
	 * 概述：第三方登录请求。
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class ThirdPartyLoginReq : public AuthBaseReq {
	public:

		/** （必填）合作伙伴平台，由UDB分配，如xl/sina/qq/*/
		string source;
		/** （必填）合作伙伴下的子系统，由UDB分配，比如mm*/
		string third_sub_sys;
		/** （必填）第三方互联协议类型: Oauth,Otp*/
		string request_type;
		/** （必填）合作伙伴AccessCode*/
		string tokenid;
		/** （选填）第三方系统的appId*/
		string third_appkey;
		/** （选填）第三方用户openid*/
		string partner_uid;
		/** （选填）第三方系统token scecret*/
		string token_secret;
		/** （选填）第三方系统的监权时返回URL*/
		string oauth_url;
		/** （选填）渠道标识*/
		string channel;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;
		/*Oath认证类型。当request_tpye=oauth_type时，可通过此字段进行选择性认证方式， 0--accesscode, 1--accesstoken(默认值)*/
		string oauth_type;
		/*（选填）登陆流水标识，业务自定义*/
		string flow_id;
		/** 预留字段暂时没用*/
		string reserve1;
		/** 预留字段暂时没用*/
		string reserve2;
		/** 预留字段暂时没用*/
		string reserve3;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		ThirdPartyLoginReq():source(""),third_sub_sys(""),request_type(""),tokenid(""),third_appkey(""),partner_uid(""),token_secret(""),oauth_type(""),flow_id(""),reserve1(""),reserve2(""),reserve3(""),oauth_url(""),channel(""),context("")
		{
			OP_CMD = RequestOpCmd::OP_THIRD_LOGIN;

		}

		ThirdPartyLoginReq(const AuthBaseReq* base):source(""),third_sub_sys(""),request_type(""),tokenid(""),third_appkey(""),partner_uid(""),token_secret(""),oauth_type(""),flow_id(""),reserve1(""),reserve2(""),reserve3(""),oauth_url(""),channel(""),context(""),AuthBaseReq(base)
		{
			OP_CMD = RequestOpCmd::OP_THIRD_LOGIN;

		}

		/**
		 * 构造一个第三方登录的请求，其它选填字段，请直接给对象赋值
		 * @param source	（必填）合作伙伴平台，由UDB分配，如xl/sina/qq/
		 * @param third_sub_sys	（必填）合作伙伴下的子系统，由UDB分配，比如mm
		 * @param request_type	（必填）第三方互联协议类型: Oauth,Otp
		 * @param tokenid	（必填）合作伙伴AccessCode
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		ThirdPartyLoginReq(string source, string third_sub_sys, string request_type, string tokenid, string oauthtype, string flowid, string context, string reserve1 = "", string reserve2 = "", string reserve3 = "") {
			OP_CMD = RequestOpCmd::OP_THIRD_LOGIN;
			this->source = source;
			this->third_sub_sys = third_sub_sys;
			this->request_type = request_type;
			this->tokenid = tokenid;
			this->context = context;
			this->oauth_type = oauthtype;
			this->flow_id = flowid;
			this->reserve1 = reserve1;
			this->reserve2 = reserve2;
			this->reserve3 = reserve3;
		}

		string marshall() {
			mJsonObject["source"] = CSJson::Value(source);
            mJsonObject["third_sub_sys"] = CSJson::Value(third_sub_sys);
			mJsonObject["tokenid"] = CSJson::Value(tokenid);
            mJsonObject["third_appkey"] = CSJson::Value(third_appkey);
			mJsonObject["partner_uid"] = CSJson::Value(partner_uid);
            mJsonObject["token_secret"] = CSJson::Value(token_secret);
			mJsonObject["oauth_url"] = CSJson::Value(oauth_url);
            mJsonObject["request_type"] = CSJson::Value(request_type);
			mJsonObject["context"] = CSJson::Value(context);
			mJsonObject["oauth_type"] = CSJson::Value(oauth_type);
			mJsonObject["flow_id"] = CSJson::Value(flow_id);
			mJsonObject["reserve1"] = CSJson::Value(reserve1);
			mJsonObject["reserve2"] = CSJson::Value(reserve2);
			mJsonObject["reserve3"] = CSJson::Value(reserve3);
			mRootJsonObject["3rdtoken_login"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {


			if(!mRootJsonObject.isMember("stru_3rdtoken_login")){
				return;
			}
			mJsonObject = mRootJsonObject["stru_3rdtoken_login"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if(mJsonObject.isMember("source") && !mJsonObject["source"].asString().empty()){
				source = mJsonObject["source"].asString();
			}
			if(mJsonObject.isMember("third_sub_sys") && !mJsonObject["third_sub_sys"].asString().empty()){
				third_sub_sys = mJsonObject["third_sub_sys"].asString();
			}
			if(mJsonObject.isMember("tokenid") && !mJsonObject["tokenid"].asString().empty()){
				tokenid = mJsonObject["tokenid"].asString();
			}
			if(mJsonObject.isMember("third_appkey") && !mJsonObject["third_appkey"].asString().empty()){
				third_appkey = mJsonObject["third_appkey"].asString();
			}
			if(mJsonObject.isMember("partner_uid") && !mJsonObject["partner_uid"].asString().empty()){
				partner_uid = mJsonObject["partner_uid"].asString();
			}
			if(mJsonObject.isMember("token_secret") && !mJsonObject["token_secret"].asString().empty()){
				token_secret = mJsonObject["token_secret"].asString();
			}
			if (mJsonObject.isMember("oauth_type") && !mJsonObject["oauth_type"].asString().empty()) {
				oauth_type = mJsonObject["oauth_type"].asString();
			}
			if (mJsonObject.isMember("flow_id") && !mJsonObject["flow_id"].asString().empty()) {
				flow_id = mJsonObject["flow_id"].asString();
			}
			if(mJsonObject.isMember("oauth_url") && !mJsonObject["oauth_url"].asString().empty()){
				oauth_url = mJsonObject["oauth_url"].asString();
			}
			if(mJsonObject.isMember("reserve1") && !mJsonObject["reserve1"].asString().empty()){
				reserve1 = mJsonObject["reserve1"].asString();
			}
			if(mJsonObject.isMember("reserve2") && !mJsonObject["reserve2"].asString().empty()){
				reserve2 = mJsonObject["reserve2"].asString();
			}
			if(mJsonObject.isMember("reserve3") && !mJsonObject["reserve3"].asString().empty()){
				reserve3 = mJsonObject["reserve3"].asString();
			}
			if(mJsonObject.isMember("request_type") && !mJsonObject["request_type"].asString().empty()){
				request_type = mJsonObject["request_type"].asString();
			}
			if(mJsonObject.isMember("channel") && !mJsonObject["channel"].asString().empty()){
				channel = mJsonObject["channel"].asString();
			}
		}
	};

	/**
	 * 概述：“手机+短信验证码”注册并登录（如果没有注册，后端会自动注册，并返回登录信息）
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class SmsRegloginReq : public AuthBaseReq {
	public:

		/** 手机号码*/
		string mobile;
		/** 短信验证码*/
		string sms_code;
		/** sha1密码，可以填空。 填空时服务器为用户设置随即密码，下次登录只能用手机短信登录*/
		string password;
		/** 填"1"时，表示只注册不登录*/
		string type;

		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		SmsRegloginReq():mobile(""), sms_code(""), context(""), password(""), type(""){
			OP_CMD = RequestOpCmd::OP_SMS_LOGIN;
		}

		SmsRegloginReq(const AuthBaseReq* base):mobile(""), sms_code(""), context(""), password(""), type(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_SMS_LOGIN;
		}

		/**
		 * 构造一个手机短信登录的请求
		 * @param mobile	手机号码 
		 * @param sms_code	短信验证码
		 * @param password	sha1密码，可以填空。 填空时服务器为用户设置随即密码，下次登录只能用手机短信登录
		 * @param type		填"1"时，表示只注册不登录
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		SmsRegloginReq(string mobile, string sms_code, string password, string type, string context) {
			OP_CMD = RequestOpCmd::OP_SMS_LOGIN;
			this->mobile = mobile;
			this->sms_code = sms_code;
			this->password = password;
			this->type = type;
			this->context = context;
		}

		string marshall() {
			mJsonObject["mobile"] = CSJson::Value(mobile);
            mJsonObject["sms_code"] = CSJson::Value(sms_code);
            mJsonObject["password"] = CSJson::Value(password);
			mJsonObject["type"] = CSJson::Value(type);
            mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["sms_reglogin"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("sms_reglogin")){
				return;
			}
			mJsonObject = mRootJsonObject["sms_reglogin"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if(mJsonObject.isMember("mobile") && !mJsonObject["mobile"].asString().empty()){
				mobile = mJsonObject["mobile"].asString();
			}

			if ( mJsonObject.isMember("sms_code") && !mJsonObject["sms_code"].asString().empty()){
				sms_code = mJsonObject["sms_code"].asString();
			}
			if ( mJsonObject.isMember("password") && !mJsonObject["password"].asString().empty()){
				password = mJsonObject["password"].asString();
			}
			if ( mJsonObject.isMember("type") && !mJsonObject["type"].asString().empty()){
				type = mJsonObject["type"].asString();
			}
		}
	};

	/**
	 * 概述：通过短信验证码修改密码
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class SmsModPwdReq : public AuthBaseReq {
	public:
		/** 手机号码*/
		string mobile;
		/** 账号，若user为空时，则修改mobile作为登录手机的账号密码。若user非空，则修改user的帐号密码*/
		string user;
		/** 短信验证码*/
		string sms_code;
		/** 新密码，SHA1加密*/
		string password;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		SmsModPwdReq():mobile(""),user(""), password(""),  sms_code(""),context("") {
			OP_CMD = RequestOpCmd::OP_SMS_MOD_PWD;
		}

		SmsModPwdReq(const AuthBaseReq* base):mobile(""),user(""),password(""),  sms_code(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_SMS_MOD_PWD;
		}

		/**
		 * 构造一个修改密码的请求
		 * @param moblie	手机号码
		 * @param user      账号，若user为空时，则修改mobile作为登录手机的账号密码。若user非空，则修改user的帐号密码
		 * @param pwd		新密码，SHA1加密
		 * @param sms_code	短信验证码
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		SmsModPwdReq(string mobile, string user, string pwd, string sms_code, string context) {
			OP_CMD = RequestOpCmd::OP_SMS_MOD_PWD;
			this->mobile = mobile;
			this->user = user;
			this->password = pwd;
			this->sms_code = sms_code;
			this->context = context;
		}

		string marshall() {
			mJsonObject["mobile"] = CSJson::Value(mobile);
			mJsonObject["user"] = CSJson::Value(user);
            mJsonObject["sms_code"] = CSJson::Value(sms_code);
			mJsonObject["pwd"] = CSJson::Value(password);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["sms_modpawd"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("sms_modpawd")){
				return;
			}
			mJsonObject = mRootJsonObject["sms_modpawd"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if(!mJsonObject.isMember("mobile") || mJsonObject["mobile"].asString().empty()){
				return;
			}
			mobile = mJsonObject["mobile"].asString();

			if(mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
				user = mJsonObject["user"].asString();
			}

			if (!mJsonObject.isMember("sms_code") || mJsonObject["sms_code"].asString().empty()){
				return;
			}
			sms_code = mJsonObject["sms_code"].asString();

			if (!mJsonObject.isMember("pwd") || mJsonObject["pwd"].asString().empty()){
				return;
			}
			password = mJsonObject["pwd"].asString();
		}
	};

	/**
	 * 概述：快速密码（一般在检验短信验证码成功之后调用）
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class QuickModPwdReq : public AuthBaseReq {
	public:

		/** 手机号码*/
		string user;
		/** 新密码，SHA1加密*/
		string password;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		QuickModPwdReq():user(""),password(""),context("") {
			OP_CMD = RequestOpCmd::OP_QUICK_MOD_PWD;
		}

		QuickModPwdReq(const AuthBaseReq* base):user(""),password(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_QUICK_MOD_PWD;
		}


		/**
		 * 构造一个快速改密请求
		 * @param user	手机号码
		 * @param password		新密码，SHA1加密
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		QuickModPwdReq(string user, string password, string context) {
			OP_CMD = RequestOpCmd::OP_QUICK_MOD_PWD;
			this->user = user;
			this->password = password;
			this->context = context;
		}
		
		string marshall() {
			mJsonObject["user"] = CSJson::Value(user);
			mJsonObject["pwd"] = CSJson::Value(password);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["quick_modpwd"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

            if(!mRootJsonObject.isMember("quick_modpwd")){
                return;
            }
            mJsonObject = mRootJsonObject["quick_modpwd"];

            if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
                return;
            }
            context = mJsonObject["context"].asString();

            if (mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
            	user = mJsonObject["user"].asString();
            }

            if (mJsonObject.isMember("pwd") && !mJsonObject["pwd"].asString().empty()){
            	password = mJsonObject["pwd"].asString();
            }
		}
	};

	/**
	 * 概述：下发手机短信
	 * @Event 对应回调事件{@link AuthEvent.SendSmsEvent}
	 */
	class SendSmsReq : public AuthBaseReq {
	public:

		/** 发下手机短信的类型：用于注册或登录（没有注册进行注册，已注册就登录）*/
		static const int SMS_TYPE_REGISTER_OR_LOGIN = 1;
		/** 发下手机短信的类型：用于登录或改密（手机号未注册，返回用户不存在）*/
		static const int SMS_TYPE_LOGIN_OR_MODPWD = 2;

		/** 用户类型：0表示{@link #user}字段是通行证，YY号，邮箱，手机号之一；*/
		static const int USER_TYPE_USERNAME = 0;
		/** 用户类型：1表示{@link #user}字段是UID*/
		static const int USER_TYPE_UID = 1;

		/** 通行证，YY号，邮箱，手机号之一（或UID）*/
		string user;
		/** 密保手机，若user不是登录手机号，则要填写密保手机。主要用于修改密码界面*/
		string sec_mobile;
		/** 用户名类型：0表示{@link #user}字段是通行证，YY号，邮箱，手机号之一；1表示{@link #user}字段是UID*/
		int user_type;
		/** 下发手机短信的类型，见TYPE_开头常量*/
		int sms_type;
		/** 二次验证策略类型，见{@link STRATEGY}类声明常量 */
		int strategy ;
		/** 二次验证码，如：图片验证就是图片验证码、其它类似*/
		string stra_token;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		SendSmsReq():user(""),sec_mobile(""),user_type(0),sms_type(SMS_TYPE_REGISTER_OR_LOGIN),strategy( STRATEGY::NONE),stra_token(""),context("") {
			OP_CMD = RequestOpCmd::OP_SEND_SMS;
		}

		SendSmsReq(const AuthBaseReq* base):user(""),sec_mobile(""),user_type(0),sms_type(SMS_TYPE_REGISTER_OR_LOGIN),strategy( STRATEGY::NONE),stra_token(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_SEND_SMS;
		}

		/**
		 * 构造一个下发短信的请求
		 * @param user	通行证，YY号，邮箱，手机号之一（或UID）
		 * @param sec_mobile	密保手机，若user不是登录手机号，则要填写密保手机。主要用于修改密码界面
		 * @param user_type	用户名类型：0表示第一个参数{@code user}是通行证，YY号，邮箱，手机号之一；1表示第一个参数{@code user}是UID
		 * @param sms_type	下发手机短信的类型，见TYPE_开头常量
		 * @param strategy	用户所选择的二次验证策略的类型
		 * @param stra_token	二次验证码，如：图片验证就是图片验证码。当为STRATEGY::NONE，这参数将被忽略
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		SendSmsReq(string user, string sec_mobile, int user_type, int sms_type, int strategy, string stra_token, string context) {
			OP_CMD = RequestOpCmd::OP_SEND_SMS;
			this->user = user;
			this->sec_mobile = sec_mobile;
			this->user_type = user_type;
			this->sms_type = sms_type;
			this->strategy = strategy;
			this->stra_token = stra_token;
			this->context = context;
		}
		
		string marshall() {
            char strusertype[10] = {0};
            sprintf(strusertype, "%d", user_type);
            char strsmstype[10] = {0};
            sprintf(strsmstype, "%d", sms_type);
            char str_strategy[10] = {0};
            sprintf(str_strategy, "%d", strategy);

			mJsonObject["user"] = CSJson::Value(user);
			mJsonObject["sec_mobile"] = CSJson::Value(sec_mobile);
			mJsonObject["user_type"] = CSJson::Value(strusertype);
			mJsonObject["type"] = CSJson::Value(strsmstype);
			mJsonObject["strategy"] = CSJson::Value(str_strategy);
			mJsonObject["stra_token"] = CSJson::Value(stra_token);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["send_sms"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("send_sms")){
				return;
			}
			mJsonObject = mRootJsonObject["send_sms"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if(mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
				user = mJsonObject["user"].asString();
			}

			if(mJsonObject.isMember("sec_mobile") && !mJsonObject["sec_mobile"].asString().empty()){
				sec_mobile = mJsonObject["sec_mobile"].asString();
			}

			if(mJsonObject.isMember("user_type") && !mJsonObject["user_type"].asString().empty()){
				user_type = atoi(mJsonObject["user_type"].asString().c_str());
			}

			if(mJsonObject.isMember("type") && !mJsonObject["type"].asString().empty()){
				sms_type = atoi(mJsonObject["type"].asString().c_str());
			}

            if(mJsonObject.isMember("strategy") && !mJsonObject["strategy"].asString().empty()){
				strategy = atoi(mJsonObject["strategy"].asString().c_str());
            }

            if(mJsonObject.isMember("stra_token") && !mJsonObject["stra_token"].asString().empty()){
            	stra_token = mJsonObject["stra_token"].asString();
            }
		}
	};

	/**
	 * 概述：检查后端是否已经收到上行短信了
	 * @Event 对应回调事件{@link AuthEvent.LoginEvent}
	 */
	class CheckSmsUpReq : public AuthBaseReq {
	public:
		/** 通行证，YY号，邮箱，手机号之一*/
		string user;
		/** 密码，SHA1加密*/
		string password ;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		CheckSmsUpReq():user(""),password(""),context(""){
			OP_CMD = RequestOpCmd::OP_CHECK_SMS_UP;

		}
		
		CheckSmsUpReq(const AuthBaseReq* base):user(""),password(""),context(""),AuthBaseReq(base){
			OP_CMD = RequestOpCmd::OP_CHECK_SMS_UP;

		}


		/**
		 * 构造一个请求
		 * @param user		通行证，YY号，邮箱，手机号之一
		 * @param password	密码，SHA1加密
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		CheckSmsUpReq(string user, string password, string context) {
			OP_CMD = RequestOpCmd::OP_CHECK_SMS_UP;
			this->user = user;
			this->password = password;
			this->context = context;
		}

		string marshall() {
			mJsonObject["user"] = CSJson::Value(user);
			mJsonObject["password"] = CSJson::Value(password);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["check_smsup"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {


			if(!mRootJsonObject.isMember("check_smsup")){
				return;
			}
			mJsonObject = mRootJsonObject["check_smsup"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if (mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
				user = mJsonObject["user"].asString();
			}
			if (mJsonObject.isMember("password") && !mJsonObject["password"].asString().empty()){
				password = mJsonObject["password"].asString();
			}
		}
	};

	/**
	 * 概述：校验手机（或账号）和短信验证码是合正确的请求
	 * @Event 对应回调事件{@link AuthEvent.VerifySmsCodeEvent}
	 */
	class VerifySmsCodeReq : public AuthBaseReq {
	public:

		/** 手机号码或用户账号*/
		string user;
		/** 短信验证码*/
		string smscode ;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		VerifySmsCodeReq():user(""),smscode(""),context("") {
			OP_CMD = RequestOpCmd::OP_VERIFY_SMSCODE;
		}

		VerifySmsCodeReq(const AuthBaseReq* base):user(""),smscode(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_VERIFY_SMSCODE;
		}


		/**
		 * 构造一个请求
		 * @param user		手机号码或用户账号
		 * @param smscode	短信验证码
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		VerifySmsCodeReq(string user, string smscode, string context) {
			OP_CMD = RequestOpCmd::OP_VERIFY_SMSCODE;
			this->user = user;
			this->smscode = smscode;
			this->context = context;
		}
		
		string marshall() {
			mJsonObject["user"] = CSJson::Value(user);
			mJsonObject["smscode"] = CSJson::Value(smscode);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["verify_smscode"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

            if(!mRootJsonObject.isMember("verify_smscode")){
                return;
            }
            mJsonObject = mRootJsonObject["verify_smscode"];

            if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
                return;
            }
            context = mJsonObject["context"].asString();

            if (mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
                user = mJsonObject["user"].asString();
            }

            if (mJsonObject.isMember("smscode") && !mJsonObject["smscode"].asString().empty()){
            	smscode = mJsonObject["smscode"].asString();
            }
		}
	};

	/**
	 * 概述：刷新图片验证码请求
	 * @Event 对应回调事件{@link AuthEvent.RefreshPicEvent}
	 */
	class RefreshPicReq : public AuthBaseReq {
	public:

		/** 通行证，YY号，邮箱，手机号之一*/
		string user;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		RefreshPicReq():user(""),context("") {
			OP_CMD = RequestOpCmd::OP_REFRESH_PIC;
		}

		RefreshPicReq(const AuthBaseReq* base):user(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_REFRESH_PIC;
		}


		/**
		 * 构造一个刷新图片验证码的请求
		 * @param user 通行证，YY号，邮箱，手机号之一
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		RefreshPicReq(string user, string context) {
			OP_CMD = RequestOpCmd::OP_REFRESH_PIC;
			this->user = user;
			this->context = context;
		}
		
		string marshall() {
			mJsonObject["user"] = CSJson::Value(user);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["refresh_pic"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("refresh_pic")){
				return;
			}
			mJsonObject = mRootJsonObject["refresh_pic"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if (mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
				user = mJsonObject["user"].asString();
			}
		}
	};


	/**
	 * 概述：查询请求
	 * @Event 对应回调事件{@link AuthEvent.QueryEvent}
	 */
	class QueryReq : public AuthBaseReq {
	public:

		/** uid*/
		unsigned long long uid;
		/** appid*/
		string appid;
		/** product_id*/
		string product_id;
		/** 获取参数*/
		string type;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		QueryReq():uid(0),appid(""),product_id(""),type(""),context("") {
			OP_CMD = RequestOpCmd::OP_QUERY;
		}

		QueryReq(const AuthBaseReq* base):uid(0),appid(""),product_id(""),type(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_QUERY;
		}

		string marshall() {
			ostringstream oss;
			oss<<uid;
			std::string struid=oss.str();

			mJsonObject["uid"] = CSJson::Value(struid);
			mJsonObject["appid"] = CSJson::Value(appid);
			mJsonObject["product_id"] = CSJson::Value(product_id);
			mJsonObject["type"] = CSJson::Value(type);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["query"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("query")){
				return;
			}
			mJsonObject = mRootJsonObject["query"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if (mJsonObject.isMember("appid") && !mJsonObject["appid"].asString().empty()){
				appid = mJsonObject["appid"].asString();
			}
			if (mJsonObject.isMember("product_id") && !mJsonObject["product_id"].asString().empty()){
				product_id = mJsonObject["product_id"].asString();
			}
			if (mJsonObject.isMember("type") && !mJsonObject["type"].asString().empty()){
				type = mJsonObject["type"].asString();
			}
			if( mJsonObject.isMember("uid") && !mJsonObject["uid"].asString().empty()){
				istringstream iss(mJsonObject["uid"].asString().c_str());
	            iss >> std::setbase(10) >> uid;
			}

		}
	};

	/**
	 * 概述：取消请求请求
	 * 没有结果
	 */
	class CancelReq : public AuthBaseReq {
	public:
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		CancelReq():context("") {
			OP_CMD = RequestOpCmd::OP_CANCEL;
		}

		CancelReq(const AuthBaseReq* base):context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_CANCEL;
		}

		CancelReq(string context) {
			OP_CMD = RequestOpCmd::OP_CANCEL;
			this->context = context;
		}
		
		string marshall() {
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["cancel_req"]=mJsonObject;
			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {
			if(!mRootJsonObject.isMember("cancel_req")){
				return;
			}
			mJsonObject = mRootJsonObject["cancel_req"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();
		}
	};

	/**
	 * 概述：改密检测请求
	 * @Event 对应回调事件{@link AuthEvent.check_modpwd_res}
	 */
	class CheckModPwdReq : public AuthBaseReq {
	public:

		/** 通行证，YY号，邮箱，手机号之一*/
		string user;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		CheckModPwdReq():user(""),context("") {
			OP_CMD = RequestOpCmd::OP_CHECK_MODPWD;
		}

		CheckModPwdReq(const AuthBaseReq* base):user(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_CHECK_MODPWD;
		}


		/**
		 * 构造一个改密检测请求
		 * @param user 通行证，YY号，邮箱，手机号之一
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		CheckModPwdReq(string user, string context) {
			OP_CMD = RequestOpCmd::OP_CHECK_MODPWD;
			this->user = user;
			this->context = context;
		}
		
		string marshall() {
			mJsonObject["user"] = CSJson::Value(user);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["check_modpwd"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("check_modpwd")){
				return;
			}
			mJsonObject = mRootJsonObject["check_modpwd"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if (mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
				user = mJsonObject["user"].asString();
			}
		}
	};

	/**
	 * 概述：注册检测请求
	 * @Event 对应回调事件{@link AuthEvent.check_reg_res}
	 */
	class CheckRegReq : public AuthBaseReq {
	public:

		/** 通行证，YY号，邮箱，手机号之一*/
		string user;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		CheckRegReq():user(""),context("") {
			OP_CMD = RequestOpCmd::OP_CHECK_REG;
		}

		CheckRegReq(const AuthBaseReq* base):user(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_CHECK_REG;
		}


		/**
		 * 构造一个注册检测请求
		 * @param user 通行证，YY号，邮箱，手机号之一
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		CheckRegReq(string user, string context) {
			OP_CMD = RequestOpCmd::OP_CHECK_REG;
			this->user = user;
			this->context = context;
		}
		
		string marshall() {
			mJsonObject["user"] = CSJson::Value(user);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["check_reg"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("check_reg")){
				return;
			}
			mJsonObject = mRootJsonObject["check_reg"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if (mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
				user = mJsonObject["user"].asString();
			}
		}
	};

	/**
	 * 概述：用户检测请求
	 * @Event 对应回调事件{@link AuthEvent.check_user_res}
	 */
	class CheckUserReq : public AuthBaseReq {
	public:

		/** 通行证，YY号，邮箱，手机号之一*/
		string user;
		/** 上下文。请求和响应的上下文是呼应的。用户在请求的时候设置该上下文，就可以与响应的上下文做比较，看是不是发出自己发出请求*/
		string context;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		CheckUserReq():user(""),context("") {
			OP_CMD = RequestOpCmd::OP_CHECK_USER;
		}

		CheckUserReq(const AuthBaseReq* base):user(""),context(""),AuthBaseReq(base) {
			OP_CMD = RequestOpCmd::OP_CHECK_USER;
		}


		/**
		 * 构造一个用户检测请求
		 * @param user 通行证，YY号，邮箱，手机号之一
		 * @param context 上下文。请求和响应的上下文是呼应的。这里设置什么值，响应的时候就是什么值
		 */
		CheckUserReq(string user, string context) {
			OP_CMD = RequestOpCmd::OP_CHECK_USER;
			this->user = user;
			this->context = context;
		}
		
		string marshall() {
			mJsonObject["user"] = CSJson::Value(user);
			mJsonObject["context"] = CSJson::Value(context);
			mRootJsonObject["check_user"]=mJsonObject;

			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {

			if(!mRootJsonObject.isMember("check_user")){
				return;
			}
			mJsonObject = mRootJsonObject["check_user"];

    		if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
    			return;
    		}
    		context = mJsonObject["context"].asString();

			if (mJsonObject.isMember("user") && !mJsonObject["user"].asString().empty()){
				user = mJsonObject["user"].asString();
			}
		}
	};

	class Exchange3rdTokenReq : public AuthBaseReq{
	public:
		/*上下文，必填*/
		std::string context;
		/*appid，必填*/
		std::string	appid;
		/*appkey，必填*/
		std::string	appkey;
		/*appid签名，非必填*/
		std::string	appidsign;
		/*票据，必填*/
		std::string	token;  
		/*票据类型：1:accesscode;2:accesstoken;3:otp*/
		uint32_t	token_type;
		/*第三方标识，必填*/
		std::string	ssid;
		/*保留字段，非必填*/
		std::string	reserve;
		CSJson::Value mJsonObject;
		Exchange3rdTokenReq():context(""),appid(""),appkey(""),appidsign(""),token(""),token_type(0),ssid(""),reserve("")
		{
			OP_CMD = RequestOpCmd::OP_CHECK_THIRDTOKEN;
		}
		Exchange3rdTokenReq(const AuthBaseReq* base):context(""),appid(""),appkey(""),appidsign(""),token(""),token_type(0),ssid(""),reserve(""),AuthBaseReq(base)
		{
			OP_CMD = RequestOpCmd::OP_CHECK_THIRDTOKEN;
		}
		Exchange3rdTokenReq(std::string context, std::string appid, std::string appkey, std::string appidsign, std::string token, uint32_t token_type, std::string ssid, std::string reserve)
		{
			OP_CMD = RequestOpCmd::OP_CHECK_THIRDTOKEN;
			this->context = context;
			this->appid = appid;
			this->appkey = appkey;
			this->appidsign = appidsign;
			this->token = token;
			this->token_type = token_type;
			this->ssid = ssid;
			this->reserve = reserve;
		}
		std::string marshall() {
			mJsonObject["context"] = CSJson::Value(context);
			mJsonObject["appid"] = CSJson::Value(appid);
			mJsonObject["appkey"] = CSJson::Value(appkey);
			mJsonObject["appidsign"] = CSJson::Value(appidsign);
			mJsonObject["token"] = CSJson::Value(token);
			char strtokentype[10] = {0};
			sprintf(strtokentype, "%d", token_type);
			mJsonObject["token_type"] = CSJson::Value(strtokentype);
			mJsonObject["ssid"] = CSJson::Value(ssid);
			mJsonObject["reserve"] = CSJson::Value(reserve);
			mRootJsonObject["exchange_3rd_token"] = mJsonObject;
			return AuthBaseReq::marshall();
		}
		void unmarshall(std::string jsonstr) {
			if (!mRootJsonObject.isMember("exchange_3rd_token"))
			{
				return;
			}
			mJsonObject = mRootJsonObject["exchange_3rd_token"];
			if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
				return;
			}
			context = mJsonObject["context"].asString();
			if(!mJsonObject.isMember("appid") || mJsonObject["appid"].asString().empty())
			{
				return;
			}
			appid = mJsonObject["appid"].asString();
			if(!mJsonObject.isMember("appkey") || mJsonObject["appkey"].asString().empty())
			{
				return;
			}
			appkey = mJsonObject["appkey"].asString();

			if(!mJsonObject.isMember("token") || mJsonObject["token"].asString().empty())
			{
				return;
			}
			token = mJsonObject["token"].asString();
			if(!mJsonObject.isMember("token_type") || mJsonObject["token_type"].asString().empty())
			{
				return;
			}
			token_type = atoi(mJsonObject["token_type"].asString().c_str());
			if(!mJsonObject.isMember("ssid") || mJsonObject["ssid"].asString().empty())
			{
				return;
			}
			ssid = mJsonObject["ssid"].asString();
			if(mJsonObject.isMember("appidsign"))
			{
				if (!mJsonObject["appidsign"].asString().empty())
				{
					appidsign = mJsonObject["appidsign"].asString();
				}
				else
					appidsign = "";
			}

			if(mJsonObject.isMember("reserve"))
			{
				if (!mJsonObject["reserve"].asString().empty())
				{
					reserve = mJsonObject["reserve"].asString();
				}
				else
					reserve = "";
			}
		}
	};


	class QuickGetYYListReq : public AuthBaseReq {
	public:
		string context;

		CSJson::Value mJsonObject;

		QuickGetYYListReq():context("")
		{
            OP_CMD = RequestOpCmd::OP_GETQUICK_YYLIST;
		}

		QuickGetYYListReq(const AuthBaseReq* base):context(""),AuthBaseReq(base)
		{
			OP_CMD = RequestOpCmd::OP_GETQUICK_YYLIST;
		}

		QuickGetYYListReq(string context) {
			OP_CMD = RequestOpCmd::OP_GETQUICK_YYLIST;
			this->context = context;
		}

		string marshall() {
			mJsonObject["context"] = CSJson::Value(context);

			mRootJsonObject["get_yy_list"] = mJsonObject;
         
			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr){
			if(!mRootJsonObject.isMember("get_yy_list")){
				return;
			}
			mJsonObject = mRootJsonObject["get_yy_list"];

			if(!mJsonObject.isMember("context") || mJsonObject["context"].asString().empty()){
				return;
			}
			context = mJsonObject["context"].asString();
		}
	};

	/**
	 * 概述：快速登录获取YY Ticket
	 * @Event 对应回调事件
	 */
	class QuickGetYYTicketReq : public AuthBaseReq {
	public:

		string context;

		int appid;

		int	   handler;

		/** 表示当前模块的JSON*/
		CSJson::Value mJsonObject ;

		QuickGetYYTicketReq():appid(0),handler(0),context("")
		{
			OP_CMD = RequestOpCmd::OP_GETQUICK_TICKET;
		}

	    QuickGetYYTicketReq(const AuthBaseReq* base):appid(0),handler(0),context(""),AuthBaseReq(base) {
	    	OP_CMD = RequestOpCmd::OP_GETQUICK_TICKET;
		}

		/**
		 * 构造
	
		 */
		QuickGetYYTicketReq(int appid, int handler, string context) {
	    	OP_CMD = RequestOpCmd::OP_GETQUICK_TICKET;
			this->context = context;
			this->appid = appid;
			this->handler = handler;
		}
		
		string marshall() {
			mJsonObject["context"] = CSJson::Value(context);

			char str_appid[10] = {0};
			itoa(appid, str_appid, 10);
			mJsonObject["appid"] = CSJson::Value(str_appid);

			char str_handler[10] = {0};
			itoa(handler, str_handler, 10);
			mJsonObject["handler"] = CSJson::Value(str_handler);

			mRootJsonObject["get_yy_ticket"]=mJsonObject;


			return AuthBaseReq::marshall();
		}

		void unmarshall(string jsonstr) {
			if(!mRootJsonObject.isMember("get_yy_ticket")){
				return;
			}
			mJsonObject = mRootJsonObject["get_yy_ticket"];

			if(!mJsonObject.isMember("context") ){
				return;
			}
			context = mJsonObject["context"].asString();

			if(mJsonObject.isMember("appid")){
				istringstream iss(mJsonObject["appid"].asString().c_str());
				iss >> std::setbase(10) >> appid;
			}

			if(mJsonObject.isMember("handler") ){
				istringstream iss(mJsonObject["handler"].asString().c_str());
				iss >> std::setbase(10) >> handler;
			}
		}
	};


	class AuthReqestConvertor
	{
	public:
		///** 将Json转换为结构体，使用后需要delete返回的指针*/
		static AuthBaseReq* toAuthReqObj(string json) {

			AuthBaseReq base ;
			base.unmarshall(json);

			switch (base.OP_CMD) {
				case RequestOpCmd::OP_PASSWORD_LOGIN:
				{
					AuthBaseReq* req = new LoginReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_CREDIT_LOGIN:
				{
					AuthBaseReq* req = new CreditLoginReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_TICKET_LOGIN:
				{
					AuthBaseReq* req = new TicketLoginReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_SEND_SMS:
				{
					AuthBaseReq* req = new SendSmsReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_SMS_LOGIN:
				{
					AuthBaseReq* req = new SmsRegloginReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_SMS_MOD_PWD:
				{
					AuthBaseReq* req = new SmsModPwdReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_REFRESH_PIC:
				{
					AuthBaseReq* req = new RefreshPicReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_QUERY:
				{
					AuthBaseReq* req = new QueryReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_QUICK_MOD_PWD:
				{
					AuthBaseReq* req = new QuickModPwdReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_CHECK_SMS_UP:
				{
					AuthBaseReq* req = new CheckSmsUpReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_VERIFY_SMSCODE:
				{
					AuthBaseReq* req = new VerifySmsCodeReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_THIRD_LOGIN:
				{
					AuthBaseReq* req = new ThirdPartyLoginReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_QRCODE_LOGIN:
				{
					AuthBaseReq* req = new QrcodeLoginReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_CANCEL:
				{
					AuthBaseReq* req = new CancelReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_CHECK_MODPWD:
				{
					AuthBaseReq* req = new CheckModPwdReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_CHECK_REG:
				{
					AuthBaseReq* req = new CheckRegReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_CHECK_USER:
				{
					AuthBaseReq* req = new CheckUserReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_CHECK_THIRDTOKEN:
				{
					AuthBaseReq* req = new Exchange3rdTokenReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_GETQUICK_YYLIST:
				{
					AuthBaseReq* req = new QuickGetYYListReq(&base);
					req->unmarshall(json);
					return req;
				}
				case RequestOpCmd::OP_GETQUICK_TICKET:
				{
					AuthBaseReq* req = new QuickGetYYTicketReq(&base);
					req->unmarshall(json);
					return req;
				}
				default:
				{
					return NULL;
				}
			}

		}
	};


}





#endif
